@@ -28,14 +28,17 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "legacy/query_mapper.h"
 #include "../util/merge_sort.h"
 #include "extend.h"
+#include "../util/algo/radix_sort.h"
 
-using namespace std;
+using std::get;
+using std::tuple;
+using std::unique_ptr;
 
 DpStat dp_stat;
 
 struct Align_fetcher
 {
-	static void init(size_t qbegin, size_t qend, vector<hit>::iterator begin, vector<hit>::iterator end)
+	static void init(size_t qbegin, size_t qend, hit* begin, hit* end)
 	{
 		it_ = begin;
 		end_ = end;
@@ -50,8 +53,7 @@ struct Align_fetcher
 			++it_;
 		end = it_;
 		this->query = query;
-		target_parallel = (end - begin > config.query_parallel_limit) && ((config.frame_shift != 0 && align_mode.mode == Align_mode::blastx && config.toppercent < 100 && config.query_range_culling)
-			|| config.ext == Config::banded_swipe);
+		target_parallel = (end - begin > config.query_parallel_limit) && (config.frame_shift == 0 || (config.toppercent < 100 && config.query_range_culling));
 		return target_parallel;
 	}
 	bool get()
@@ -63,36 +65,31 @@ struct Align_fetcher
 			queue_->release();
 	}
 	size_t query;
-	vector<hit>::iterator begin, end;
+	hit* begin, *end;
 	bool target_parallel;
 private:	
-	static vector<hit>::iterator it_, end_;
+	static hit* it_, *end_;
 	static unique_ptr<Queue> queue_;
 };
 
 unique_ptr<Queue> Align_fetcher::queue_;
-vector<hit>::iterator Align_fetcher::it_;
-vector<hit>::iterator Align_fetcher::end_;
+hit* Align_fetcher::it_;
+hit* Align_fetcher::end_;
 
 TextBuffer* legacy_pipeline(Align_fetcher &hits, const sequence *subjects, size_t subject_count, const Metadata *metadata, const Parameters *params, Statistics &stat) {
 	if ((hits.end == hits.begin) && subjects == nullptr) {
 		TextBuffer *buf = nullptr;
 		if (!blocked_processing && *output_format != Output_format::daa && config.report_unaligned != 0) {
 			buf = new TextBuffer;
-			const char *query_title = query_ids::get()[hits.query].c_str();
+			const char *query_title = query_ids::get()[hits.query];
 			output_format->print_query_intro(hits.query, query_title, get_source_query_len((unsigned)hits.query), *buf, true);
 			output_format->print_query_epilog(*buf, query_title, true, *params);
 		}
 		return buf;
 	}
 
-	QueryMapper *mapper;
-	if (config.ext == Config::swipe)
-		mapper = new ExtensionPipeline::Swipe::Pipeline(*params, hits.query, hits.begin, hits.end, *metadata);
-	else if (config.frame_shift != 0 || config.ext == Config::banded_swipe)
-		mapper = new ExtensionPipeline::BandedSwipe::Pipeline(*params, hits.query, hits.begin, hits.end, dp_stat, *metadata, hits.target_parallel);
-	else
-		mapper = new ExtensionPipeline::Greedy::Pipeline(*params, hits.query, hits.begin, hits.end, *metadata);
+	QueryMapper *mapper = new ExtensionPipeline::BandedSwipe::Pipeline(*params, hits.query, hits.begin, hits.end, dp_stat, *metadata, hits.target_parallel);
+
 	task_timer timer("Initializing mapper", hits.target_parallel ? 3 : UINT_MAX);
 	mapper->init();
 	timer.finish();
@@ -119,13 +116,14 @@ void align_worker(size_t thread_id, const Parameters *params, const Metadata *me
 	Statistics stat;
 	DpStat dp_stat;
 	while (hits.get()) {
-		if(config.ext != Config::banded_swipe) {
+		if(config.frame_shift != 0) {
 			TextBuffer *buf = legacy_pipeline(hits, subjects, subject_count, metadata, params, stat);
 			OutputSink::get().push(hits.query, buf);
 			hits.release();
 			continue;
 		}
-		vector<Extension::Match> matches = Extension::extend(*params, hits.query, hits.begin, hits.end, *metadata, stat, hits.target_parallel ? Extension::TARGET_PARALLEL : 0);
+		task_timer timer;
+		vector<Extension::Match> matches = Extension::extend(*params, hits.query, hits.begin, hits.end, *metadata, stat, hits.target_parallel ? DP::PARALLEL : 0);
 		TextBuffer *buf = Extension::generate_output(matches, hits.query, stat, *metadata, *params);
 		if (!matches.empty() && (!config.unaligned.empty() || !config.aligned_file.empty())) {
 			query_aligned_mtx.lock();
@@ -133,6 +131,8 @@ void align_worker(size_t thread_id, const Parameters *params, const Metadata *me
 			query_aligned_mtx.unlock();
 		}
 		OutputSink::get().push(hits.query, buf);
+		if (hits.target_parallel)
+			stat.inc(Statistics::TIME_TARGET_PARALLEL, timer.microseconds());
 		hits.release();
 	}
 	statistics += stat;
@@ -141,7 +141,9 @@ void align_worker(size_t thread_id, const Parameters *params, const Metadata *me
 
 void align_queries(Trace_pt_buffer &trace_pts, Consumer* output_file, const Parameters &params, const Metadata &metadata)
 {
-	const size_t max_size = (size_t)std::min(config.chunk_size*1e9 * 9 * 2 / config.lowmem, 2e9);
+	size_t max_size = std::min(size_t(config.chunk_size*1e9 * 10 * 2) / config.lowmem / 3, config.trace_pt_fetch_size);
+	if (config.memory_limit != 0.0)
+		max_size = std::max(max_size, size_t(config.memory_limit * 1e9));
 	pair<size_t, size_t> query_range;
 	vector<sequence> subjects;
 	if (config.swipe_all) {
@@ -150,39 +152,40 @@ void align_queries(Trace_pt_buffer &trace_pts, Consumer* output_file, const Para
 			subjects.push_back(ref_seqs::get()[i]);
 	}
 
+	trace_pts.load(max_size);
+
 	while (true) {
 		task_timer timer("Loading trace points", 3);
-		Trace_pt_list *v = new Trace_pt_list;
-		statistics.max(Statistics::TEMP_SPACE, trace_pts.load(*v, max_size, query_range));
-		if (query_range.second - query_range.first == 0) {
-			delete v;
+		tuple<vector<hit>*, size_t, size_t> input = trace_pts.retrieve();
+		if (get<0>(input) == nullptr)
 			break;
-		}
+		statistics.inc(Statistics::TIME_LOAD_SEED_HITS, timer.microseconds());
+		vector<hit>* hit_buf = get<0>(input);
+		query_range = { get<1>(input), get<2>(input) };
+		trace_pts.load(max_size);
+
 		timer.go("Sorting trace points");
-		merge_sort(v->begin(), v->end(), config.threads_);
-		v->init();
+		//if (config.beta)
+			radix_sort<hit, hit::Query>(hit_buf->data(), hit_buf->data() + hit_buf->size(), (uint32_t)query_range.second * align_mode.query_contexts, config.threads_);
+		//else
+			//merge_sort(hit_buf->begin(), hit_buf->end(), config.threads_);
+		statistics.inc(Statistics::TIME_SORT_SEED_HITS, timer.microseconds());
+
 		timer.go("Computing alignments");
-		Align_fetcher::init(query_range.first, query_range.second, v->begin(), v->end());
+		Align_fetcher::init(query_range.first, query_range.second, hit_buf->data(), hit_buf->data() + hit_buf->size());
 		OutputSink::instance = unique_ptr<OutputSink>(new OutputSink(query_range.first, output_file));
-		vector<thread> threads;
-		if (config.verbosity >= 3 && config.load_balancing == Config::query_parallel)
+		vector<std::thread> threads;
+		if (config.verbosity >= 3 && config.load_balancing == Config::query_parallel && !config.no_heartbeat)
 			threads.emplace_back(heartbeat_worker, query_range.second);
 		size_t n_threads = config.load_balancing == Config::query_parallel ? (config.threads_align == 0 ? config.threads_ : config.threads_align) : 1;
 		for (size_t i = 0; i < n_threads; ++i)
 			threads.emplace_back(align_worker, i, &params, &metadata, subjects.empty() ? nullptr : subjects.data(), subjects.size());
 		for (auto &t : threads)
 			t.join();
-		timer.finish();
-
-		double t = timer.get();
-		log_stream << "Gross cells = " << dp_stat.gross_cells << endl;
-		log_stream << "Gross GCUPS = " << (double)dp_stat.gross_cells / 1e9 / t << endl;
-		log_stream << "Gross GCUPS/thread = " << (double)dp_stat.gross_cells / n_threads / 1e9 / t << endl;
-		log_stream << "Net cells = " << dp_stat.net_cells << endl;
-		log_stream << "Net GCUPS = " << (double)dp_stat.net_cells / 1e9 / t << endl;
-		log_stream << "Net GCUPS/thread = " << (double)dp_stat.net_cells / n_threads / 1e9 / t << endl;
-
+		statistics.inc(Statistics::TIME_EXT, timer.microseconds());
+		
 		timer.go("Deallocating buffers");
-		delete v;
+		delete hit_buf;
 	}
+	statistics.max(Statistics::SEARCH_TEMP_SPACE, trace_pts.total_disk_size());
 }
\ No newline at end of file
