@@ -60,12 +60,12 @@ struct Target : public ::Target
 				d1 = std::min(i->diagonal() + band, d_max);
 			}
 			else {
-				v.emplace_back(subject, d0, d1, target_idx);
+				v.emplace_back(subject, d0, d1, 0, 0, target_idx);
 				d0 = std::max(i->diagonal() - band, d_min);
 				d1 = std::min(i->diagonal() + band, d_max);
 			}
 		}
-		v.emplace_back(subject, d0, d1, target_idx);
+		v.emplace_back(subject, d0, d1, 0, 0, target_idx);
 	}
 
 	void add(QueryMapper &mapper, vector<DpTarget> &vf, vector<DpTarget> &vr, int target_idx)
@@ -127,7 +127,7 @@ void Pipeline::range_ranking()
 {
 	const double rr = config.rank_ratio == -1 ? 0.4 : config.rank_ratio;
 	std::stable_sort(targets.begin(), targets.end(), Target::compare);
-	IntervalPartition ip((int)std::min(config.max_alignments, (uint64_t)INT_MAX));
+	IntervalPartition ip((int)std::min(config.max_alignments, (size_t)INT_MAX));
 	for (PtrVector< ::Target>::iterator i = targets.begin(); i < targets.end();) {
 		Target* t = ((Target*)*i);
 		if (t->is_outranked(ip, source_query_len, rr)) {
@@ -156,13 +156,8 @@ void Pipeline::run_swipe(bool score_only)
 	for (size_t i = 0; i < n_targets(); ++i)
 		target(i).add(*this, vf, vr, (int)i);
 	list<Hsp> hsp;
-	if (score_matrix.frame_shift()) {
-		hsp = banded_3frame_swipe(translated_query, FORWARD, vf.begin(), vf.end(), this->dp_stat, score_only, target_parallel);
-		hsp.splice(hsp.end(), banded_3frame_swipe(translated_query, REVERSE, vr.begin(), vr.end(), this->dp_stat, score_only, target_parallel));
-	}
-	else {
-		hsp = DP::BandedSwipe::swipe(query_seq(0), vf.begin(), vf.end(), Frame(0), config.comp_based_stats ? &query_cb[0] : nullptr, score_only ? 0 : DP::TRACEBACK, raw_score_cutoff());
-	}
+	hsp = banded_3frame_swipe(translated_query, FORWARD, vf.begin(), vf.end(), this->dp_stat, score_only, target_parallel);
+	hsp.splice(hsp.end(), banded_3frame_swipe(translated_query, REVERSE, vr.begin(), vr.end(), this->dp_stat, score_only, target_parallel));
 	
 	while (!hsp.empty()) {
 		list<Hsp> &l = target(hsp.begin()->swipe_target).hsps;
@@ -170,7 +165,7 @@ void Pipeline::run_swipe(bool score_only)
 	}
 }
 
-void build_ranking_worker(PtrVector<::Target>::iterator begin, PtrVector<::Target>::iterator end, atomic<size_t> *next, vector<unsigned> *intervals) {
+void build_ranking_worker(PtrVector<::Target>::iterator begin, PtrVector<::Target>::iterator end, atomic<size_t> *next, vector<int32_t> *intervals) {
 	PtrVector<::Target>::iterator it;
 	while ((it = begin + next->fetch_add(64)) < end) {
 		auto e = min(it + 64, end);
@@ -210,9 +205,9 @@ void Pipeline::run(Statistics &stat, const sequence *subjects, size_t subject_co
 
 		if (target_parallel) {
 			timer.go("Building score ranking intervals");
-			vector<vector<unsigned>> intervals(config.threads_);
+			vector<vector<int32_t>> intervals(config.threads_);
 			const size_t interval_count = (source_query_len + ::Target::INTERVAL - 1) / ::Target::INTERVAL;
-			for (vector<unsigned> &v : intervals)
+			for (vector<int32_t> &v : intervals)
 				v.resize(interval_count);
 			vector<thread> threads;
 			atomic<size_t> next(0);
@@ -223,7 +218,7 @@ void Pipeline::run(Statistics &stat, const sequence *subjects, size_t subject_co
 
 			timer.go("Merging score ranking intervals");
 			for (auto it = intervals.begin() + 1; it < intervals.end(); ++it) {
-				vector<unsigned>::iterator i = intervals[0].begin(), i1 = intervals[0].end(), j = it->begin();
+				vector<int32_t>::iterator i = intervals[0].begin(), i1 = intervals[0].end(), j = it->begin();
 				for (; i < i1; ++i, ++j)
 					*i = max(*i, *j);
 			}
