@@ -20,7 +20,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <algorithm>
 #include "query_mapper.h"
 #include "../../data/reference.h"
-#include "../extend_ungapped.h"
+#include "../../dp/ungapped.h"
 #include "../../output/output.h"
 #include "../../output/output_format.h"
 #include "../../output/daa_write.h"
@@ -52,7 +52,7 @@ bool Target::is_enveloped(PtrVector<Target>::const_iterator begin, PtrVector<Tar
 	return false;
 }
 
-void Target::add_ranges(vector<unsigned> &v) {
+void Target::add_ranges(vector<int32_t> &v) {
 	for (const Hsp &hsp : hsps) {
 		const int i0 = hsp.query_source_range.begin_ / INTERVAL,
 			i1 = min(hsp.query_source_range.end_ / INTERVAL, int(v.size() - 1));
@@ -61,12 +61,12 @@ void Target::add_ranges(vector<unsigned> &v) {
 	}
 }
 
-bool Target::is_outranked(const vector<unsigned> &v, double treshold) {
+bool Target::is_outranked(const vector<int32_t> &v, double treshold) {
 	for (const Hsp &hsp : hsps) {
 		const int i0 = hsp.query_source_range.begin_ / INTERVAL,
 			i1 = min(hsp.query_source_range.end_ / INTERVAL, int(v.size() - 1));
 		for (int i = i0; i <= i1; ++i)
-			if (hsp.score >= unsigned(v[i] * treshold))
+			if (hsp.score >= v[i] * treshold)
 				return false;
 	}
 	return true;
@@ -77,7 +77,7 @@ int QueryMapper::raw_score_cutoff() const
 	return score_matrix.rawscore(config.min_bit_score == 0 ? score_matrix.bitscore(config.max_evalue, (unsigned)query_seq(0).length()) : config.min_bit_score);
 }
 
-QueryMapper::QueryMapper(const Parameters &params, size_t query_id, Trace_pt_list::iterator begin, Trace_pt_list::iterator end, const Metadata &metadata, bool target_parallel) :
+QueryMapper::QueryMapper(const Parameters &params, size_t query_id, hit* begin, hit* end, const Metadata &metadata, bool target_parallel) :
 	parameters(params),
 	source_hits(std::make_pair(begin, end)),
 	query_id((unsigned)query_id),
@@ -94,14 +94,10 @@ QueryMapper::QueryMapper(const Parameters &params, size_t query_id, Trace_pt_lis
 void QueryMapper::init()
 {
 	if(config.log_query)
-		cout << "Query = " << query_ids::get()[query_id].c_str() << endl;
+		cout << "Query = " << query_ids::get()[query_id] << '\t' << query_id << endl;
 	if (config.comp_based_stats == 1)
 		for (unsigned i = 0; i < align_mode.query_contexts; ++i)
 			query_cb.emplace_back(query_seq(i));
-	if (config.ext == Config::greedy || config.ext == Config::more_greedy)
-		for (unsigned i = 0; i < align_mode.query_contexts; ++i)
-			profile.emplace_back(query_seq(i));
-			//profile.push_back(Long_score_profile());
 	targets.resize(count_targets());
 	if (targets.empty())
 		return;
@@ -110,13 +106,13 @@ void QueryMapper::init()
 
 unsigned QueryMapper::count_targets()
 {
-	std::sort(source_hits.first, source_hits.second, hit::cmp_subject);
+	std::sort(source_hits.first, source_hits.second, hit::CmpSubject());
 	const size_t n = source_hits.second - source_hits.first;
-	const Trace_pt_list::iterator hits = source_hits.first;
+	const hit* hits = source_hits.first;
 	size_t subject_id = std::numeric_limits<size_t>::max();
 	unsigned n_subject = 0;
 	for (size_t i = 0; i < n; ++i) {
-		std::pair<size_t, size_t> l = ref_seqs::data_->local_position(hits[i].subject_);
+		std::pair<size_t, size_t> l = ref_seqs::data_->local_position((uint64_t)hits[i].subject_);
 		const unsigned frame = hits[i].query_ % align_mode.query_contexts;
 		/*const Diagonal_segment d = config.comp_based_stats ? xdrop_ungapped(query_seq(frame), query_cb[frame], ref_seqs::get()[l.first], hits[i].seed_offset_, (int)l.second)
 			: xdrop_ungapped(query_seq(frame), ref_seqs::get()[l.first], hits[i].seed_offset_, (int)l.second);*/
@@ -129,7 +125,7 @@ unsigned QueryMapper::count_targets()
 		}
 		else {
 			const Diagonal_segment d = xdrop_ungapped(query_seq(frame), ref_seqs::get()[l.first], hits[i].seed_offset_, (int)l.second);
-			if (d.score >= config.min_ungapped_raw_score) {
+			if (d.score > 0) {
 				if (l.first != subject_id) {
 					subject_id = l.first;
 					++n_subject;
@@ -170,11 +166,11 @@ void QueryMapper::rank_targets(double ratio, double factor)
 		score = int((double)targets[0].filter_score * (1.0 - config.toppercent / 100.0) * ratio);
 	}
 	else {
-		size_t min_idx = std::min(targets.size(), (size_t)config.max_alignments);
+		size_t min_idx = std::min(targets.size(), config.max_alignments);
 		score = int((double)targets[min_idx - 1].filter_score * ratio);
 	}
 
-	const size_t cap = (config.toppercent < 100 || config.max_alignments == std::numeric_limits<uint64_t>::max()) ? std::numeric_limits<uint64_t>::max() : size_t(config.max_alignments*factor);
+	const size_t cap = (config.toppercent < 100 || config.max_alignments == std::numeric_limits<size_t>::max()) ? std::numeric_limits<size_t>::max() : size_t(config.max_alignments*factor);
 	size_t i = 0;
 	for (; i < targets.size(); ++i)
 		if (targets[i].filter_score < score || i >= cap)
@@ -222,7 +218,7 @@ bool QueryMapper::generate_output(TextBuffer &buffer, Statistics &stat)
 	unique_ptr<TargetCulling> target_culling(TargetCulling::get());
 	const unsigned query_len = (unsigned)query_seq(0).length();
 	size_t seek_pos = 0;
-	const char *query_title = query_ids::get()[query_id].c_str();
+	const char *query_title = query_ids::get()[query_id];
 	unique_ptr<Output_format> f(output_format->clone());
 
 	for (size_t i = 0; i < targets.size(); ++i) {
@@ -234,7 +230,7 @@ bool QueryMapper::generate_output(TextBuffer &buffer, Statistics &stat)
 		const size_t subject_id = targets[i].subject_block_id;
 		const unsigned database_id = ReferenceDictionary::get().block_to_database_id(subject_id);
 		const unsigned subject_len = (unsigned)ref_seqs::get()[subject_id].length();
-		const char *ref_title = ref_ids::get()[subject_id].c_str();
+		const char *ref_title = ref_ids::get()[subject_id];
 		targets[i].apply_filters(source_query_len, subject_len, query_title, ref_title);
 		if (targets[i].hsps.size() == 0)
 			continue;
