@@ -1,8 +1,8 @@
 /****
 DIAMOND protein aligner
 Copyright (C) 2013-2020 Max Planck Society for the Advancement of Science e.V.
-						Benjamin Buchfink
-						Eberhard Karls Universitaet Tuebingen
+                        Benjamin Buchfink
+                        Eberhard Karls Universitaet Tuebingen
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -38,7 +38,7 @@ TextBuffer* generate_output(vector<Match> &targets, size_t query_block_id, Stati
 	unsigned n_hsp = 0, hit_hsps = 0;
 	TranslatedSequence query = query_seqs::get().translated_seq(align_mode.query_translated ? query_source_seqs::get()[query_block_id] : query_seqs::get()[query_block_id], query_block_id*align_mode.query_contexts);
 	const unsigned query_len = (unsigned)query.index(0).length();
-	const char *query_title = query_ids::get()[query_block_id].c_str();
+	const char *query_title = query_ids::get()[query_block_id];
 	const bool aligned = !targets.empty();
 
 	if (blocked_processing) {
@@ -49,13 +49,13 @@ TextBuffer* generate_output(vector<Match> &targets, size_t query_block_id, Stati
 		} else if(aligned || config.report_unaligned)
 			f->print_query_intro(query_block_id, query_title, query.source().length(), *out, !aligned);
 	}
-	
+		
 	for (size_t i = 0; i < targets.size(); ++i) {
 
 		const size_t subject_id = targets[i].target_block_id;
 		const unsigned database_id = ReferenceDictionary::get().block_to_database_id(subject_id);
 		const unsigned subject_len = (unsigned)ref_seqs::get()[subject_id].length();
-		const char *ref_title = ref_ids::get()[subject_id].c_str();
+		const char *ref_title = ref_ids::get()[subject_id];
 
 		if (targets[i].outranked)
 			stat.inc(Statistics::OUTRANKED_HITS);
@@ -79,7 +79,8 @@ TextBuffer* generate_output(vector<Match> &targets, size_t query_block_id, Stati
 						subject_len,
 						i,
 						hit_hsps,
-						ref_seqs::get()[subject_id]), metadata, *out);
+						ref_seqs::get()[subject_id],
+						targets[i].ungapped_score), metadata, *out);
 			}
 
 			++n_hsp;
