@@ -1,6 +1,10 @@
 /****
 DIAMOND protein aligner
-Copyright (C) 2013-2019 Benjamin Buchfink <buchfink@gmail.com>
+Copyright (C) 2013-2020 Max Planck Society for the Advancement of Science e.V.
+                        Benjamin Buchfink
+                        Eberhard Karls Universitaet Tuebingen
+
+Code developed by Benjamin Buchfink <benjamin.buchfink@tue.mpg.de>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -17,6 +21,11 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ****/
 
 #include <memory>
+#include <cstdlib>
+#include <sys/stat.h>
+#include <exception>
+#include <iomanip>
+#include <numeric>
 #include "../util/command_line_parser.h"
 #include "config.h"
 #include "../util/util.h"
@@ -28,17 +37,80 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "shape_config.h"
 #include "../util/io/temp_file.h"
 #include "../basic/match.h"
+#include "../cluster/cluster_registry.h"
 #include "../basic/translate.h"
 #include "../dp/dp.h"
 #include "masking.h"
 #include "../util/system/system.h"
 #include "../util/simd.h"
+#include "../util/parallel/multiprocessing.h"
 
 using namespace std;
 
 Config config;
 
-Config::Config(int argc, const char **argv)
+void print_warnings() {
+	if (config.sensitivity >= Sensitivity::VERY_SENSITIVE || config.verbosity == 0)
+		return;
+	const double ram = total_ram();
+	unsigned b = 2, c = 4;
+	stringstream msg;
+	if (config.sensitivity >= Sensitivity::SENSITIVE) {
+		if (ram >= 63) {
+			c = 1;
+			if(c < config.lowmem)
+				msg << "use this parameter for better performance: -c1";
+		}
+	}
+	else {
+		if (ram >= 511) {
+			b = 12;
+			c = 1;
+		}
+		else if (ram >= 255) {
+			b = 8;
+			c = 1;
+		}
+		else if (ram >= 127) {
+			b = 5;
+			c = 1;
+		}
+		else if (ram >= 63) {
+			b = 6;
+		}
+		else if (ram >= 31) {
+			b = 4;
+		}
+		if ((b > 2 && b > config.chunk_size) || c < config.lowmem) {
+			msg << "increase the block size for better performance using these parameters : -b" << b;
+			if (c != 4)
+				msg << " -c" << c;
+		}
+	}
+	if (!msg.str().empty()) {
+		set_color(Color::YELLOW, true);
+		cerr << "The host system is detected to have " << (size_t)ram << " GB of RAM. It is recommended to " << msg.str() << endl;
+		reset_color(true);
+	}
+}
+
+template<typename _t>
+_t set_string_option(const string& s, const string& name, const vector<pair<string, _t>>& values) {
+	if (s.empty())
+		return (_t)0;
+	for (auto& i : values)
+		if (s == i.first)
+			return i.second;
+	throw std::runtime_error("Invalid argument for option " + name + ". Allowed values are:" + std::accumulate(values.begin(), values.end(), string(), [](const string& s, const pair<string, _t>& v) { return s + ' ' + v.first; }));
+}
+
+void Config::set_sens(Sensitivity sens) {
+	if (sensitivity != Sensitivity::FAST)
+		throw std::runtime_error("Sensitivity switches are mutually exclusive.");
+	sensitivity = sens;
+}
+
+Config::Config(int argc, const char **argv, bool check_io)
 {
 	Command_line_parser parser;
 	parser.add_command("makedb", "Build DIAMOND database from a FASTA file")
@@ -68,7 +140,6 @@ Config::Config(int argc, const char **argv)
 		.add_command("info", "")
 		.add_command("seed-stat", "")
 		.add_command("smith-waterman", "")
-		.add_command("protein-snps", "")
 		.add_command("cluster", "")
 		.add_command("translate", "")
 		.add_command("filter-blasttab", "")
@@ -77,10 +148,13 @@ Config::Config(int argc, const char **argv)
 		.add_command("split", "")
 		.add_command("upgma", "")
 		.add_command("upgmamc", "")
-		.add_command("test", "")
+		.add_command("test", "Run regression tests")
 		.add_command("reverse", "")
 		.add_command("compute-medoids", "")
-		.add_command("mutate", "");
+		.add_command("mutate", "")
+		.add_command("merge-tsv", "");
+
+	string traceback_mode_str;
 
 	Options_group general("General options");
 	general.add()
@@ -129,6 +203,7 @@ Config::Config(int argc, const char **argv)
 \tstitle means Subject Title\n\
 \tsalltitles means All Subject Title(s), separated by a '<>'\n\
 \tqcovhsp means Query Coverage Per HSP\n\
+\tscovhsp means Subject Coverage Per HSP\n\
 \tqtitle means Query title\n\
 \tqqual means Query quality values for the aligned part of the query\n\
 \tfull_qqual means Query quality values\n\
@@ -141,7 +216,18 @@ Config::Config(int argc, const char **argv)
 
 	Options_group makedb("Makedb options");
 	makedb.add()
-		("in", 0, "input reference file in FASTA format", input_ref_file);
+		("in", 0, "input reference file in FASTA format", input_ref_file)
+		("taxonmap", 0, "protein accession to taxid mapping file", prot_accession2taxid)
+		("taxonnodes", 0, "taxonomy nodes.dmp from NCBI", nodesdmp)
+		("taxonnames", 0, "taxonomy names.dmp from NCBI", namesdmp);
+
+	Options_group cluster("");
+	cluster.add()
+		("cluster-algo", 0, "Clustering algorithm (\"multi-step\", \"mcl\")", cluster_algo)
+		("cluster-similarity", 0, "Clustering similarity measure", cluster_similarity)
+		("mcl-expansion", 0, "MCL expansion coefficient (default=2)", cluster_mcl_expansion, (uint32_t) 2)
+		("mcl-inflation", 0, "MCL inflation coefficient (default=2.0)", cluster_mcl_inflation, 2.0)
+		("mcl-sparsity-switch", 0, "MCL switch to sparse matrix computation (default=0.8) ", cluster_mcl_sparsity_switch, 0.8);
 
 	Options_group aligner("Aligner options");
 	aligner.add()
@@ -152,8 +238,9 @@ Config::Config(int argc, const char **argv)
 		("unfmt", 0, "format of unaligned query file (fasta/fastq)", unfmt, string("fasta"))
 		("alfmt", 0, "format of aligned query file (fasta/fastq)", alfmt, string("fasta"))
 		("unal", 0, "report unaligned queries (0=no, 1=yes)", report_unaligned, -1)
-		("max-target-seqs", 'k', "maximum number of target sequences to report alignments for", max_alignments, uint64_t(25))
+		("max-target-seqs", 'k', "maximum number of target sequences to report alignments for (default=25)", max_alignments, size_t(25))
 		("top", 0, "report alignments within this percentage range of top alignment score (overrides --max-target-seqs)", toppercent, 100.0)
+		("max-hsps", 0, "maximum number of HSPs per target sequence to report for each query (default=1)", max_hsps, 1u)
 		("range-culling", 0, "restrict hit culling to overlapping query ranges", query_range_culling)
 		("compress", 0, "compression for output files (0=none, 1=gzip)", compression)
 		("evalue", 'e', "maximum e-value to report alignments (default=0.001)", max_evalue, 0.001)
@@ -161,11 +248,15 @@ Config::Config(int argc, const char **argv)
 		("id", 0, "minimum identity% to report an alignment", min_id)
 		("query-cover", 0, "minimum query cover% to report an alignment", query_cover)
 		("subject-cover", 0, "minimum subject cover% to report an alignment", subject_cover)
+		("mid-sensitive", 0, "enable mid-sensitive mode (default: fast)", mode_mid_sensitive)
 		("sensitive", 0, "enable sensitive mode (default: fast)", mode_sensitive)
 		("more-sensitive", 0, "enable more sensitive mode (default: fast)", mode_more_sensitive)
+		("very-sensitive", 0, "enable very sensitive mode (default: fast)", mode_very_sensitive)
+		("ultra-sensitive", 0, "enable ultra sensitive mode (default: fast)", mode_ultra_sensitive)
 		("block-size", 'b', "sequence block size in billions of letters (default=2.0)", chunk_size)
-		("index-chunks", 'c', "number of chunks for index processing", lowmem)
+		("index-chunks", 'c', "number of chunks for index processing (default=4)", lowmem)
 		("tmpdir", 't', "directory for temporary files", tmpdir)
+		("parallel-tmpdir", 0, "directory for temporary files used by multiprocessing", parallel_tmpdir)
 		("gapopen", 0, "gap open penalty", gap_open, -1)
 		("gapextend", 0, "gap extension penalty", gap_extend, -1)
 		("frameshift", 'F', "frame shift penalty (default=disabled)", frame_shift)
@@ -176,44 +267,39 @@ Config::Config(int argc, const char **argv)
 		("K", 0, "K parameter for custom matrix", K)
 		("comp-based-stats", 0, "enable composition based statistics (0/1=default)", comp_based_stats, 1u)
 		("masking", 0, "enable masking of low complexity regions (0/1=default)", masking, 1)
-		//("seg", 0, "enable SEG masking of queries (yes/no)", seg)
 		("query-gencode", 0, "genetic code to use to translate query (see user manual)", query_gencode, 1u)
 		("salltitles", 0, "include full subject titles in DAA file", salltitles)
 		("sallseqid", 0, "include all subject ids in DAA file", sallseqid)
 		("no-self-hits", 0, "suppress reporting of identical self hits", no_self_hits)
-		("taxonmap", 0, "protein accession to taxid mapping file", prot_accession2taxid)
-		("taxonnodes", 0, "taxonomy nodes.dmp from NCBI", nodesdmp)
-		("taxonnames", 0, "taxonomy names.dmp from NCBI", namesdmp)
 		("taxonlist", 0, "restrict search to list of taxon ids (comma-separated)", taxonlist)
 		("taxon-exclude", 0, "exclude list of taxon ids (comma-separated)", taxon_exclude);
 
 	Options_group advanced("Advanced options");
 	advanced.add()
 		("algo", 0, "Seed search algorithm (0=double-indexed/1=query-indexed)", algo, -1)
-		("bin", 0, "number of query bins for seed search", query_bins, 16u)
+		("bin", 0, "number of query bins for seed search", query_bins)
 		("min-orf", 'l', "ignore translated sequences without an open reading frame of at least this length", run_len)
 		("freq-sd", 0, "number of standard deviations for ignoring frequent seeds", freq_sd, 0.0)
 		("id2", 0, "minimum number of identities for stage 1 hit", min_identities)
-		("window", 'w', "window size for local hit search", window)
 		("xdrop", 'x', "xdrop for ungapped alignment", ungapped_xdrop, 12.3)
-		("ungapped-score", 0, "minimum alignment score to continue local extension", min_ungapped_score)
-		("hit-band", 0, "band for hit verification", hit_band)
-		("hit-score", 0, "minimum score to keep a tentative alignment", min_hit_score)
-		("gapped-xdrop", 'X', "xdrop for gapped alignment in bits", gapped_xdrop, 20)
 		("band", 0, "band for dynamic programming computation", padding)
-		("shapes", 's', "number of seed shapes (0 = all available)", shapes)
+		("shapes", 's', "number of seed shapes (default=all available)", shapes)
 		("shape-mask", 0, "seed shapes", shape_mask)
-		("index-mode", 0, "index mode (0=4x12, 1=16x9)", index_mode)
-		("rank-ratio", 0, "include subjects within this ratio of last hit (stage 1)", rank_ratio, -1.0)
-		("rank-ratio2", 0, "include subjects within this ratio of last hit (stage 2)", rank_ratio2, -1.0)
-		("max-hsps", 0, "maximum number of HSPs per subject sequence to save for each query", max_hsps, 0u)
+		("multiprocessing", 0, "enable distributed-memory parallel processing", multiprocessing)
+		("mp-init", 0, "initialize multiprocessing run", mp_init)
+		("ext-chunk-size", 0, "chunk size for adaptive ranking (default=auto)", ext_chunk_size)
+		("no-ranking", 0, "disable ranking heuristic", no_ranking)
+		("ext", 0, "Extension mode (banded-fast/banded-slow)", ext)
+		("culling-overlap", 0, "minimum range overlap with higher scoring hit to delete a hit (default=50%)", inner_culling_overlap, 50.0)
 		("taxon-k", 0, "maximum number of targets to report per species", taxon_k, (uint64_t)0)
-		("range-cover", 0, "percentage of query range to be covered for hit culling (default=50)", query_range_cover, 50.0)
+		("range-cover", 0, "percentage of query range to be covered for range culling (default=50%)", query_range_cover, 50.0)
 		("dbsize", 0, "effective database size (in letters)", db_size)
 		("no-auto-append", 0, "disable auto appending of DAA and DMND file extensions", no_auto_append)
 		("xml-blord-format", 0, "Use gnl|BL_ORD_ID| style format in XML output", xml_blord_format)
 		("stop-match-score", 0, "Set the match score of stop codons against each other.", stop_match_score, 1)
-		("tantan-minMaskProb", 0, "minimum repeat probability for masking (0.9)", tantan_minMaskProb, 0.9);
+		("tantan-minMaskProb", 0, "minimum repeat probability for masking (default=0.9)", tantan_minMaskProb, 0.9)
+		("file-buffer-size", 0, "file buffer size in bytes (default=67108864)", file_buffer_size, (size_t)67108864)
+		("memory-limit", 'M', "Memory limit for extension stage in GB", memory_limit);
 
 	Options_group view_options("View options");
 	view_options.add()
@@ -224,6 +310,18 @@ Config::Config(int argc, const char **argv)
 	getseq_options.add()
 		("seq", 0, "Sequence numbers to display.", seq_no);
 
+	double rank_ratio2;
+	unsigned window, min_ungapped_score, hit_band, min_hit_score;
+	Options_group deprecated_options("");
+	deprecated_options.add()
+		("window", 'w', "window size for local hit search", window)
+		("ungapped-score", 0, "minimum alignment score to continue local extension", min_ungapped_score)
+		("hit-band", 0, "band for hit verification", hit_band)
+		("hit-score", 0, "minimum score to keep a tentative alignment", min_hit_score)
+		("gapped-xdrop", 'X', "xdrop for gapped alignment in bits", gapped_xdrop, 20)
+		("rank-ratio2", 0, "include subjects within this ratio of last hit (stage 2)", rank_ratio2, -1.0)
+		("rank-ratio", 0, "include subjects within this ratio of last hit", rank_ratio, -1.0);
+
 	Options_group hidden_options("");
 	hidden_options.add()
 		("extend-all", 0, "extend all seed hits", extend_all)
@@ -239,7 +337,6 @@ Config::Config(int argc, const char **argv)
 		("reverse", 0, "", reverse)
 		("neighborhood-score", 0, "", neighborhood_score)
 		("seed-weight", 'w', "", seed_weight, 7u)
-		("very-sensitive", 0, "", mode_very_sensitive)
 		("idl", 0, "", id_left)
 		("idr", 0, "", id_right)
 		("idn", 0, "", id_n)
@@ -260,7 +357,6 @@ Config::Config(int argc, const char **argv)
 		("superblock", 0, "", superblock, 128)
 		("max-cells", 0, "", max_cells, 10000000u)
 		("load-balancing", 0, "", load_balancing, (unsigned)Config::query_parallel)
-		("ext", 0, "", ext, (int)Config::greedy)
 		("br", 0, "", benchmark_ranking)
 		("log-query", 0, "", log_query)
 		("log-subject", 0, "", log_subject)
@@ -272,7 +368,6 @@ Config::Config(int argc, const char **argv)
 		("hashed-seeds", 0, "", hashed_seeds)
 		("rank-factor", 0, "", rank_factor, -1.0)
 		("transcript-len-estimate", 0, "", transcript_len_estimate, 1.0)
-		("no-traceback", 0, "", disable_traceback)
 		("family-counts", 0, "", family_counts_file)
 		("radix-cluster-buffered", 0, "", radix_cluster_buffered)
 		("join-split-size", 0, "", join_split_size, 100000u)
@@ -286,7 +381,7 @@ Config::Config(int argc, const char **argv)
 		("use-dataset-field", 0, "", use_dataset_field)
 		("store-query-quality", 0, "", store_query_quality)
 		("swipe-chunk-size", 0, "", swipe_chunk_size, 256u)
-		("query-parallel-limit", 0, "", query_parallel_limit, 1000000u)
+		("query-parallel-limit", 0, "", query_parallel_limit, 3000000u)
 		("hard-masked", 0, "", hardmasked)
 		("cbs-window", 0, "", cbs_window, 40)
 		("no-unlink", 0, "", no_unlink)
@@ -300,11 +395,67 @@ Config::Config(int argc, const char **argv)
 		("chaining-maxgap", 0, "", chaining_maxgap, 2000)
 		("tantan-maxRepeatOffset", 0, "maximum tandem repeat period to consider (50)", tantan_maxRepeatOffset, 15)
 		("tantan-ungapped", 0, "use tantan masking in ungapped mode", tantan_ungapped)
-		("family-map", 0, "", family_map);
+		("family-map", 0, "", family_map)
+		("family-map-query", 0, "", family_map_query)
+		("chaining-range-cover", 0, "", chaining_range_cover, (size_t)8)
+		("index-mode", 0, "index mode (0=4x12, 1=16x9)", index_mode)
+		("no-swipe-realign", 0, "", no_swipe_realign)
+		("cut-bar", 0, "", cut_bar)
+		("bootstrap", 0, "", bootstrap)
+		("chaining-maxnodes", 0, "", chaining_maxnodes)
+		("cutoff-score-8bit", 0, "", cutoff_score_8bit, 240)
+		("min-band-overlap", 0, "", min_band_overlap, 0.2)
+		("min-realign-overhang", 0, "", min_realign_overhang, 30)
+		("ungapped-window", 0, "", ungapped_window, 48)
+		("gapped-filter-diag-score", 0, "", gapped_filter_diag_bit_score, 12.0)
+		("gapped-filter-evalue", 0, "", gapped_filter_evalue, -1.0)
+		("gapped-filter-window", 0, "", gapped_filter_window, 200)
+		("output-hits", 0, "", output_hits)
+		("ungapped-evalue", 0, "", ungapped_evalue, -1.0)
+		("no-logfile", 0, "", no_logfile)
+		("no-heartbeat", 0, "", no_heartbeat)
+		("band-bin", 0, "", band_bin, 24)
+		("col-bin", 0, "", col_bin, 400)
+		("self", 0, "", self)
+		("trace-pt-fetch-size", 0, "", trace_pt_fetch_size, (size_t)10e9)
+		("tile-size", 0, "", tile_size, (uint32_t)1024)
+		("short-query-ungapped-bitscore", 0, "", short_query_ungapped_bitscore, 25.0)
+		("short-query-max-len", 0, "", short_query_max_len, 60)
+		("gapped-filter-evalue1", 0, "", gapped_filter_evalue1, 1.0e+04)
+		("ext-yield", 0, "", ext_min_yield)
+		("full-sw-len", 0, "", full_sw_len)
+		("relaxed-evalue-factor", 0, "", relaxed_evalue_factor, 1.0)
+		("type", 0, "", type)
+		("raw", 0, "", raw)
+		("chaining-len-cap", 0, "", chaining_len_cap, 2.0)
+		("chaining-min-nodes", 0, "", chaining_min_nodes, (size_t)200)
+		("fast-tsv", 0, "", fast_tsv)
+		("target-parallel-verbosity", 0, "", target_parallel_verbosity, UINT_MAX)
+		("ext-targets", 0, "", global_ranking_targets)
+		("traceback-mode", 0, "", traceback_mode_str)
+		("query-memory", 0, "", query_memory)
+		("memory-intervals", 0, "", memory_intervals, (size_t)2)
+		("seed-hit-density", 0, "", seedhit_density)
+		("chunk-size-multiplier", 0, "", chunk_size_multiplier, (size_t)4)
+		("score-drop-factor", 0, "", ranking_score_drop_factor, 0.95)
+		("left-most-interval", 0, "", left_most_interval, 32)
+		("ranking-cutoff-bitscore", 0, "", ranking_cutoff_bitscore, 25.0);
 	
-	parser.add(general).add(makedb).add(aligner).add(advanced).add(view_options).add(getseq_options).add(hidden_options);
+	parser.add(general).add(makedb).add(cluster).add(aligner).add(advanced).add(view_options).add(getseq_options).add(hidden_options).add(deprecated_options);
 	parser.store(argc, argv, command);
 
+	traceback_mode = set_string_option<TracebackMode>(traceback_mode_str, "--traceback-mode",
+		{ {"score", TracebackMode::SCORE_ONLY },
+		{"stat", TracebackMode::STAT},
+		{"vector", TracebackMode::VECTOR},
+		{"buffer", TracebackMode::SCORE_BUFFER} });
+
+	if (toppercent != 100.0 && max_alignments != 25)
+		throw std::runtime_error("--top and --max-target-seqs are mutually exclusive.");
+
+	if (command == blastx && no_self_hits)
+		throw std::runtime_error("--no-self-hits option is not supported in blastx mode.");
+
 	if (long_reads) {
 		query_range_culling = true;
 		if (toppercent == 100.0)
@@ -313,42 +464,44 @@ Config::Config(int argc, const char **argv)
 			frame_shift = 15;
 	}
 
-	switch (command) {
-	case Config::makedb:
-		if (database == "")
-			throw std::runtime_error("Missing parameter: database file (--db/-d)");
-		if (chunk_size != 0.0)
-			throw std::runtime_error("Invalid option: --block-size/-b. Block size is set for the alignment commands.");
-		break;
-	case Config::blastp:
-	case Config::blastx:
-		if (database == "")
-			throw std::runtime_error("Missing parameter: database file (--db/-d)");
-		if (daa_file.length() > 0) {
-			if (output_file.length() > 0)
-				throw std::runtime_error("Options --daa and --out cannot be used together.");
-			if (output_format.size() > 0 && output_format[0] != "daa")
-				throw std::runtime_error("Invalid parameter: --daa/-a. Output file is specified with the --out/-o parameter.");
-			output_file = daa_file;
+	if (check_io) {
+		switch (command) {
+		case Config::makedb:
+			if (database == "")
+				throw std::runtime_error("Missing parameter: database file (--db/-d)");
+			if (chunk_size != 0.0)
+				throw std::runtime_error("Invalid option: --block-size/-b. Block size is set for the alignment commands.");
+			break;
+		case Config::blastp:
+		case Config::blastx:
+			if (database == "")
+				throw std::runtime_error("Missing parameter: database file (--db/-d)");
+			if (daa_file.length() > 0) {
+				if (output_file.length() > 0)
+					throw std::runtime_error("Options --daa and --out cannot be used together.");
+				if (output_format.size() > 0 && output_format[0] != "daa")
+					throw std::runtime_error("Invalid parameter: --daa/-a. Output file is specified with the --out/-o parameter.");
+				output_file = daa_file;
+			}
+			if (daa_file.length() > 0 || (output_format.size() > 0 && (output_format[0] == "daa" || output_format[0] == "100"))) {
+				if (compression != 0)
+					throw std::runtime_error("Compression is not supported for DAA format.");
+				if (!no_auto_append)
+					auto_append_extension(output_file, ".daa");
+			}
+			break;
+		case Config::view:
+			if (daa_file == "")
+				throw std::runtime_error("Missing parameter: DAA file (--daa/-a)");
+		default:
+			;
 		}
-		if (daa_file.length() > 0 || (output_format.size() > 0 && (output_format[0] == "daa" || output_format[0] == "100"))) {
-			if (compression != 0)
-				throw std::runtime_error("Compression is not supported for DAA format.");
-			if (!no_auto_append)
-				auto_append_extension(output_file, ".daa");
-		}
-		break;
-	case Config::view:
-		if (daa_file == "")
-			throw std::runtime_error("Missing parameter: DAA file (--daa/-a)");
-	default:
-		;
-	}
 
-	switch (command) {
-	case Config::dbinfo:
-		if (database == "")
-			throw std::runtime_error("Missing parameter: database file (--db/-d)");
+		switch (command) {
+		case Config::dbinfo:
+			if (database == "")
+				throw std::runtime_error("Missing parameter: database file (--db/-d)");
+		}
 	}
 
 	if (hit_cap != 0)
@@ -361,7 +514,7 @@ Config::Config(int argc, const char **argv)
 	else if (verbose)
 		verbosity = 2;
 	else if (((command == Config::view || command == blastx || command == blastp) && output_file == "")
-		|| command == Config::version || command == getseq || command == fastq2fasta)
+		|| command == Config::version || command == getseq || command == fastq2fasta || command == regression_test)
 		verbosity = 0;
 	else
 		verbosity = 1;
@@ -371,9 +524,9 @@ Config::Config(int argc, const char **argv)
 		message_stream = Message_stream(false);
 		break;
 	case 3:
-		log_stream = Message_stream(true, true);
-		verbose_stream = Message_stream(true, true);
-		message_stream = Message_stream(true, true);
+		log_stream = Message_stream(true, !config.no_logfile);
+		verbose_stream = Message_stream(true, !config.no_logfile);
+		message_stream = Message_stream(true, !config.no_logfile);
 		break;
 	case 2:
 		verbose_stream = Message_stream();
@@ -395,9 +548,11 @@ Config::Config(int argc, const char **argv)
 			auto_append_extension(output_file, ".gz");
 	}
 
-	ostream &header_out = command == Config::help ? cout : cerr;
-	header_out << Const::program_name << " v" << Const::version_string << "." << (unsigned)Const::build_version << " (C) Max Planck Society for the Advancement of Science" << endl;
-	header_out << "Documentation, support and updates available at http://www.diamondsearch.org" << endl << endl;
+	if (verbosity >= 1 || command == regression_test) {
+		ostream &header_out = command == Config::help ? cout : cerr;
+		header_out << Const::program_name << " v" << Const::version_string << "." << (unsigned)Const::build_version << " (C) Max Planck Society for the Advancement of Science" << endl;
+		header_out << "Documentation, support and updates available at http://www.diamondsearch.org" << endl << endl;
+	}
 	log_stream << Const::program_name << " v" << Const::version_string << "." << (unsigned)Const::build_version << endl;
 #ifndef NDEBUG
 	verbose_stream << "Assertions enabled." << endl;
@@ -410,6 +565,7 @@ Config::Config(int argc, const char **argv)
 	case Config::blastx:
 	case Config::view:
 	case Config::cluster:
+	case Config::regression_test:
 	case Config::compute_medoids:
 		message_stream << "#CPU threads: " << threads_ << endl;
 	default:
@@ -425,6 +581,7 @@ Config::Config(int argc, const char **argv)
 	case Config::mask:
 	case Config::makedb:
 	case Config::cluster:
+	case Config::regression_test:
 	case Config::compute_medoids:
 		if (frame_shift != 0 && command == Config::blastp)
 			throw std::runtime_error("Frameshift alignments are only supported for translated searches.");
@@ -439,21 +596,43 @@ Config::Config(int argc, const char **argv)
 				throw std::runtime_error("Custom scoring matrices require setting the --gapopen and --gapextend options.");
 			score_matrix = Score_matrix(matrix_file, lambda, K, gap_open, gap_extend);
 		}
+		if(command == Config::cluster && !Workflow::Cluster::ClusterRegistry::has(cluster_algo)){
+			ostream &header_out = command == Config::help ? cout : cerr;
+			header_out << "Unkown clustering algorithm: " << cluster_algo << endl;
+			header_out << "Available options are: " << endl;
+			for(string c_algo : Workflow::Cluster::ClusterRegistry::getKeys()){
+				header_out << "\t" << c_algo << "\t"<< Workflow::Cluster::ClusterRegistry::get(c_algo)->get_description() << endl;
+			}
+			throw std::runtime_error("Clustering algorithm not found.");
+		}
 		message_stream << "Scoring parameters: " << score_matrix << endl;
 		if (masking == 1)
 			Masking::instance = unique_ptr<Masking>(new Masking(score_matrix));
 	}
 
 	if (command == Config::blastp || command == Config::blastx || command == Config::benchmark || command == Config::model_sim || command == Config::opt
-		|| command == Config::mask || command == Config::cluster || command == Config::compute_medoids) {
+		|| command == Config::mask || command == Config::cluster || command == Config::compute_medoids || command == Config::regression_test) {
 		if (tmpdir == "")
 			tmpdir = extract_dir(output_file);
-		
+
 		init_cbs();
 		raw_ungapped_xdrop = score_matrix.rawscore(ungapped_xdrop);
 		verbose_stream << "CPU features detected: " << SIMD::features() << endl;
 	}
 
+	sensitivity = Sensitivity::FAST;
+	if (mode_mid_sensitive) set_sens(Sensitivity::MID_SENSITIVE);
+	if (mode_sensitive) set_sens(Sensitivity::SENSITIVE);
+	if (mode_more_sensitive) set_sens(Sensitivity::MORE_SENSITIVE);
+	if (mode_very_sensitive) set_sens(Sensitivity::VERY_SENSITIVE);
+	if (mode_ultra_sensitive) set_sens(Sensitivity::ULTRA_SENSITIVE);
+
+	if (algo == Config::query_indexed && (sensitivity == Sensitivity::MID_SENSITIVE || sensitivity >= Sensitivity::VERY_SENSITIVE))
+		throw std::runtime_error("Query-indexed mode is not supported for this sensitivity setting.");
+
+	if (ext != "banded-fast" && ext != "banded-slow" && ext != "")
+		throw std::runtime_error("Possible values for --ext are: banded-fast, banded-slow");
+
 	Translator::init(query_gencode);
 
 	if (command == blastx)
@@ -475,11 +654,41 @@ Config::Config(int argc, const char **argv)
 
 	if (swipe_all) {
 		algo = double_indexed;
-		ext = swipe;
 	}
 
 	use_lazy_dict = false;
 
 	if (query_range_culling && taxon_k != 0)
 		throw std::runtime_error("--taxon-k is not supported for --range-culling mode.");
+
+	if (parallel_tmpdir == "") {
+		parallel_tmpdir = tmpdir;
+	} else {
+#ifndef WIN32
+		if (multiprocessing) {
+			// char * env_str = std::getenv("SLURM_JOBID");
+			// if (env_str) {
+			// 	parallel_tmpdir = join_path(parallel_tmpdir, "diamond_job_"+string(env_str));
+			// }
+			errno = 0;
+			int s = mkdir(parallel_tmpdir.c_str(), 00770);
+			if (s != 0) {
+				if (errno == EEXIST) {
+					// directory did already exist
+				} else {
+					throw(std::runtime_error("could not create parallel temporary directory " + parallel_tmpdir));
+				}
+			}
+		}
+#endif
+	}
+
+	log_stream << "MAX_SHAPE_LEN=" << MAX_SHAPE_LEN;
+#ifdef SEQ_MASK
+	log_stream << " SEQ_MASK";
+#endif
+#ifdef STRICT_BAND
+	log_stream << " STRICT_BAND";
+#endif
+	log_stream << endl;
 }
