@@ -1,6 +1,10 @@
 /****
 DIAMOND protein aligner
-Copyright (C) 2013-2019 Benjamin Buchfink <buchfink@gmail.com>
+Copyright (C) 2013-2020 Max Planck Society for the Advancement of Science e.V.
+                        Benjamin Buchfink
+                        Eberhard Karls Universitaet Tuebingen
+
+Code developed by Benjamin Buchfink <benjamin.buchfink@tue.mpg.de>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -16,29 +20,29 @@ You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ****/
 
-#ifndef CONFIG_H_
-#define CONFIG_H_
-
+#pragma once
 #include <string>
 #include <vector>
 #include <stdint.h>
 
-using std::string;
-using std::vector;
+enum class Sensitivity { FAST = 0, MID_SENSITIVE = 1, SENSITIVE = 2, MORE_SENSITIVE = 3, VERY_SENSITIVE = 4, ULTRA_SENSITIVE = 5 };
+enum class TracebackMode { NONE = 0, SCORE_ONLY = 1, STAT = 2, VECTOR = 3, SCORE_BUFFER = 4 };
 
 struct Config
 {
-	string	input_ref_file;
+
+	using string = std::string;
+	using string_vector = std::vector<std::string>;
+
+	string_vector input_ref_file;
 	unsigned	threads_;
 	string	database;
 	string	query_file;
 	unsigned	merge_seq_treshold;
 	unsigned	hit_cap;
-	double min_ungapped_score;
-	int		min_ungapped_raw_score;
 	unsigned shapes;
 	unsigned	index_mode;
-	uint64_t	max_alignments;
+	size_t	max_alignments;
 	string	match_file1;
 	string	match_file2;
 	int		padding;
@@ -50,18 +54,14 @@ struct Config
 	unsigned min_identities2;
 	double ungapped_xdrop;
 	int		raw_ungapped_xdrop;
-	unsigned window;
-	double		min_hit_score;
-	int min_hit_raw_score;
-	int		hit_band;
 	unsigned	min_compressed_identities;
 	int		min_seed_score;
 	unsigned	seed_signatures;
 	double	min_bit_score;
 	unsigned	run_len;
-	bool		disable_traceback;
 	double	max_seed_freq;
 	string	tmpdir;
+	string	parallel_tmpdir;
 	bool		long_mode;
 	int		gapped_xdrop;
 	double	max_evalue;
@@ -78,7 +78,7 @@ struct Config
 	unsigned	compress_temp;
 	double	toppercent;
 	string	daa_file;
-	vector<string>	output_format;
+	string_vector	output_format;
 	string	output_file;
 	bool		forwardonly;
 	unsigned fetch_size;
@@ -91,10 +91,9 @@ struct Config
 	unsigned local_align_mode;
 	bool extend_all;
 	bool slow_search;
-	vector<string> seq_no;
-	double rank_ratio;
-	double rank_ratio2;
+	string_vector seq_no;
 	double rank_factor;
+	double rank_ratio;
 	bool ht_mode;
 	bool old_freq;
 	double freq_sd;
@@ -102,8 +101,7 @@ struct Config
 	bool mode_more_sensitive;
 	string matrix_file;
 	double lambda, K;
-	vector<string> shape_mask;
-	unsigned seed_anchor;
+	string_vector shape_mask;
 	unsigned query_gencode;
 	string unaligned;
 	double space_penalty;
@@ -190,28 +188,86 @@ struct Config
 	bool log_extend;
 	int chaining_maxgap;
 	string family_map;
+	string family_map_query;
+	size_t chaining_range_cover;
+	bool no_swipe_realign;
+	bool cut_bar;
+	bool bootstrap;
+	size_t chaining_maxnodes;
+	int cutoff_score_8bit;
+	double inner_culling_overlap;
+	double min_band_overlap;
+	int min_realign_overhang;
+	int ungapped_window;
+	int gapped_filter_diag_score;
+	double gapped_filter_diag_bit_score;
+	double gapped_filter_evalue;
+	int gapped_filter_window;
+	bool output_hits;
+	double ungapped_evalue;
+	bool no_logfile;
+	bool no_heartbeat;
+	int band_bin;
+	int col_bin;
+	size_t file_buffer_size;
+	bool self;
+	size_t trace_pt_fetch_size;
+	uint32_t tile_size;
+	double short_query_ungapped_bitscore;
+	int short_query_max_len;
+	double gapped_filter_evalue1;
+	size_t ext_chunk_size;
+	double ext_min_yield;
+	string ext;
+	int full_sw_len;
+	double relaxed_evalue_factor;
+	string type;
+	bool raw;
+	bool mode_ultra_sensitive;
+	double chaining_len_cap;
+	size_t chaining_min_nodes;
+	bool fast_tsv;
+	unsigned target_parallel_verbosity;
+	double memory_limit;
+	size_t global_ranking_targets;
+	bool mode_mid_sensitive;
+	bool no_ranking;
+	bool query_memory;
+	size_t memory_intervals;
+	size_t seedhit_density;
+	size_t chunk_size_multiplier;
+	double ranking_score_drop_factor;
+	double ranking_cutoff_bitscore;
+	int left_most_interval;
+
+	Sensitivity sensitivity;
+	TracebackMode traceback_mode;
+
+	bool multiprocessing;
+	bool mp_init;
 
 	enum {
 		makedb = 0, blastp = 1, blastx = 2, view = 3, help = 4, version = 5, getseq = 6, benchmark = 7, random_seqs = 8, compare = 9, sort = 10, roc = 11, db_stat = 12, model_sim = 13,
 		match_file_stat = 14, model_seqs = 15, opt = 16, mask = 17, fastq2fasta = 18, dbinfo = 19, test_extra = 20, test_io = 21, db_annot_stats = 22, read_sim = 23, info = 24, seed_stat = 25,
-		smith_waterman = 26, protein_snps = 27, cluster = 28, translate = 29, filter_blasttab = 30, show_cbs = 31, simulate_seqs = 32, split = 33, upgma = 34, upgma_mc = 35, regression_test = 36,
-		reverse_seqs = 37, compute_medoids = 38, mutate = 39
+		smith_waterman = 26, cluster = 27, translate = 28, filter_blasttab = 29, show_cbs = 30, simulate_seqs = 31, split = 32, upgma = 33, upgma_mc = 34, regression_test = 35,
+		reverse_seqs = 36, compute_medoids = 37, mutate = 38, merge_tsv = 39
 	};
 	unsigned	command;
 
 	enum { double_indexed = 0, query_indexed = 1, subject_indexed = 2 };
 	int algo;
 
+	string cluster_algo;
+	double cluster_mcl_inflation;
+	uint32_t cluster_mcl_expansion;
+	double cluster_mcl_sparsity_switch;
+	string	cluster_similarity;
+
 	enum { query_parallel = 0, target_parallel = 1 };
 	unsigned load_balancing;
 
-	enum {
-		swipe = 0, greedy = 1, floating_xdrop = 4, more_greedy = 2, most_greedy=3, banded_swipe=4
-	};
-	int ext;
-
 	Config() {}
-	Config(int argc, const char **argv);
+	Config(int argc, const char **argv, bool check_io = true);
 
 	inline unsigned get_run_len(unsigned length)
 	{
@@ -235,40 +291,18 @@ struct Config
 			return n_target_seq < max_alignments;
 	}
 
-	/*unsigned read_padding(size_t len)
-	{
-		if (padding == 0) {
-			if (len <= 255)
-				return 10;
-			else
-				return 32;
-		}
-		else
-			return padding;
-	}*/
-
-	unsigned read_padding(size_t len)
-	{
-		if (padding == 0) {
-			if (mode_very_sensitive)
-				return 60;
-			else if (len <= 35)
-				return 5;
-			else if (len <= 55)
-				return 16;
-			else
-				return 32;
-		}
-		else
-			return padding;
-	}
+	void set_sens(Sensitivity sens);
 
 	bool mem_buffered() const { return tmpdir == "/dev/shm"; }
 
   	template<typename _t>
-	static void set_option(_t& option, _t value) { if (option == 0) option = value; }
+	static void set_option(_t& option, _t value, _t def = 0) { if (option == def) option = value; }
 };
 
+void print_warnings();
 extern Config config;
 
-#endif
\ No newline at end of file
+template<typename _t>
+_t top_cutoff_score(_t top_score) {
+	return _t((1.0 - config.toppercent / 100.0)*top_score);
+}
\ No newline at end of file
