@@ -1,6 +1,10 @@
 /****
 DIAMOND protein aligner
-Copyright (C) 2013-2019 Benjamin Buchfink <buchfink@gmail.com>
+Copyright (C) 2013-2020 Max Planck Society for the Advancement of Science e.V.
+                        Benjamin Buchfink
+                        Eberhard Karls Universitaet Tuebingen
+						
+Code developed by Benjamin Buchfink <benjamin.buchfink@tue.mpg.de>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -16,9 +20,8 @@ You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ****/
 
-#ifndef MATCH_H_
-#define MATCH_H_
 
+#pragma once
 #include <limits>
 #include <list>
 #include "sequence.h"
@@ -53,11 +56,13 @@ struct Hsp
 		positives(0),
 		gap_openings(0),
 		gaps(0),
-		sw_score(0)
+		swipe_target(0),
+		d_begin(0),
+		d_end(0)
 	{}
 
-	Hsp(int score, unsigned swipe_target = 0) :
-		score(unsigned(score)),
+	Hsp(int score, int swipe_target = 0) :
+		score(score),
 		frame(0),
 		length(0),
 		identities(0),
@@ -65,8 +70,9 @@ struct Hsp
 		positives(0),
 		gap_openings(0),
 		gaps(0),
-		sw_score(0),
-		swipe_target(swipe_target)
+		swipe_target(swipe_target),
+		d_begin(0),
+		d_end(0)
 	{}
 
 	Hsp(const IntermediateRecord &r, unsigned query_source_len);
@@ -151,6 +157,15 @@ struct Hsp
 			query_range.begin_ = (dna_len + f - oriented_query_begin) / 3;
 	}
 
+	void set_translated_query_end(unsigned oriented_query_end, unsigned dna_len)
+	{
+		int f = frame <= 2 ? frame + 1 : 2 - frame;
+		if (f > 0)
+			query_range.end_ = (oriented_query_end - 2 - (f - 1)) / 3 + 1;
+		else
+			query_range.end_ = (dna_len + f - oriented_query_end) / 3 + 1;
+	}
+
 	int blast_query_frame() const
 	{
 		return align_mode.query_translated ? (frame <= 2 ? (int)frame + 1 : 2 - (int)frame) : 0;
@@ -158,7 +173,7 @@ struct Hsp
 
 	bool operator<(const Hsp &rhs) const
 	{
-		return score > rhs.score;
+		return score > rhs.score || (score == rhs.score && query_source_range.begin_ < rhs.query_source_range.begin_);
 	}
 
 	double id_percent() const
@@ -197,7 +212,7 @@ struct Hsp
 	bool is_weakly_enveloped_by(std::list<Hsp>::const_iterator begin, std::list<Hsp>::const_iterator end, int cutoff) const;
 	void push_back(const DiagonalSegment &d, const TranslatedSequence &query, const sequence &subject, bool reversed);
 	void push_match(Letter q, Letter s, bool positive);
-	void push_gap(Edit_operation op, int length, const char *subject);
+	void push_gap(Edit_operation op, int length, const Letter *subject);
 	void splice(const DiagonalSegment &d0, const DiagonalSegment &d1, const TranslatedSequence &query, const sequence &subject, bool reversed);
 	void set_begin(const DiagonalSegment &d, int dna_len);
 	void set_end(const DiagonalSegment &d, int dna_len);
@@ -207,9 +222,8 @@ struct Hsp
 
 	bool is_weakly_enveloped(const Hsp &j) const;
 	std::pair<int, int> diagonal_bounds() const;
-	unsigned score, frame, length, identities, mismatches, positives, gap_openings, gaps, sw_score, swipe_target;
-	float time;
-	interval query_source_range, query_range, subject_range;
+	int score, frame, length, identities, mismatches, positives, gap_openings, gaps, swipe_target, d_begin, d_end;
+	interval query_source_range, query_range, subject_range, seed_hit_range;
 	Packed_transcript transcript;
 };
 
@@ -226,7 +240,8 @@ struct Hsp_context
 		unsigned subject_len,
 		unsigned hit_num,
 		unsigned hsp_num,
-		const sequence &subject_seq
+		const sequence &subject_seq,
+		int ungapped_score = 0
 	) :
 		query(query),
 		query_name(query_name),
@@ -237,6 +252,7 @@ struct Hsp_context
 		subject_len(subject_len),
 		hit_num(hit_num),
 		hsp_num(hsp_num),
+		ungapped_score(ungapped_score),
 		subject_seq(subject_seq),
 		hsp_(hsp)		
 	{}
@@ -320,14 +336,6 @@ struct Hsp_context
 	{
 		return score_matrix.bitscore(score());
 	}
-	double sw_score() const
-	{
-		return score_matrix.bitscore(hsp_.sw_score);
-	}
-	double time() const
-	{
-		return hsp_.time;
-	}
 	unsigned frame() const
 	{ return hsp_.frame; }
 	unsigned length() const
@@ -359,9 +367,8 @@ struct Hsp_context
 	const TranslatedSequence query;
 	const char *query_name, *subject_name;
 	const unsigned query_id, subject_id, orig_subject_id, subject_len, hit_num, hsp_num;
+	int ungapped_score;
 	const sequence subject_seq;
 private:	
 	Hsp &hsp_;
 };
-
-#endif /* MATCH_H_ */
