@@ -1,6 +1,9 @@
 /****
 DIAMOND protein aligner
-Copyright (C) 2013-2018 Benjamin Buchfink <buchfink@gmail.com>
+Copyright (C) 2016-2020 Max Planck Society for the Advancement of Science e.V.
+                        Benjamin Buchfink
+						
+Code developed by Benjamin Buchfink <benjamin.buchfink@tue.mpg.de>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -16,18 +19,12 @@ You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ****/
 
-#ifndef PARAMETERS_H_
-#define PARAMETERS_H_
-
+#pragma once
 #include <stdint.h>
+#include "../util/scores/cutoff_table.h"
 
 struct Parameters
 {
-	Parameters(uint64_t db_seqs, uint64_t db_letters):
-		db_seqs(db_seqs),
-		db_letters(db_letters)
-	{}
-	const uint64_t db_seqs, db_letters;
-};
-
-#endif
\ No newline at end of file
+	const uint64_t db_seqs, db_letters, ref_blocks;
+	const Util::Scores::CutoffTable cutoff_gapped1, cutoff_gapped2;
+};
\ No newline at end of file
