@@ -25,7 +25,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 struct Seed_iterator
 {
-	Seed_iterator(vector<char> &seq, const Shape &sh):
+	Seed_iterator(vector<Letter> &seq, const Shape &sh):
 		ptr_ (seq.data()),
 		end_ (ptr_ + seq.size() - sh.length_ + 1)
 	{}
@@ -38,7 +38,7 @@ struct Seed_iterator
 		return sh.set_seed_reduced(seed, ptr_++);
 	}
 private:
-	const char *ptr_, *end_;
+	const Letter *ptr_, *end_;
 };
 
 template<uint64_t _b>
@@ -50,7 +50,11 @@ struct Hashed_seed_iterator
 		last_(0)
 	{
 		for (uint64_t i = 0; i < sh.length_ - 1; ++i)
+#ifdef SEQ_MASK
+			last_ = (last_ << _b) | Reduction::reduction(letter_mask(*(ptr_++)));
+#else
 			last_ = (last_ << _b) | Reduction::reduction(*(ptr_++));
+#endif
 	}
 	bool good() const
 	{
@@ -59,7 +63,11 @@ struct Hashed_seed_iterator
 	bool get(uint64_t &seed, uint64_t mask)
 	{
 		last_ <<= _b;
-		const char l = *(ptr_++);
+#ifdef SEQ_MASK
+		const Letter l = *(ptr_++) & LETTER_MASK;
+#else
+		const Letter l = *(ptr_++);
+#endif
 		if (l == value_traits.mask_char)
 			return false;
 		last_ |= Reduction::reduction(l);
@@ -67,7 +75,7 @@ struct Hashed_seed_iterator
 		return true;
 	}
 private:
-	const char *ptr_, *end_;
+	const Letter *ptr_, *end_;
 	uint64_t last_;
 };
 
@@ -80,7 +88,11 @@ struct Contiguous_seed_iterator
 		last_(0)
 	{
 		for (uint64_t i = 0; i < _l - 1; ++i)
+#ifdef SEQ_MASK
+			last_ = (last_ << _b) | Reduction::reduction(letter_mask(*(ptr_++)));
+#else
 			last_ = (last_ << _b) | Reduction::reduction(*(ptr_++));
+#endif
 	}
 	bool good() const
 	{
@@ -90,7 +102,11 @@ struct Contiguous_seed_iterator
 	{
 		last_ <<= _b;
 		last_ &= (1 << (_b*_l)) - 1;
-		const char l = *(ptr_++);
+#ifdef SEQ_MASK
+		const Letter l = *(ptr_++) & LETTER_MASK;
+#else
+		const Letter l = *(ptr_++);
+#endif
 		if (l == value_traits.mask_char)
 			return false;
 		last_ |= Reduction::reduction(l);
@@ -102,7 +118,7 @@ struct Contiguous_seed_iterator
 		return _l;
 	}
 private:
-	const char *ptr_, *end_;
+	const Letter *ptr_, *end_;
 	uint64_t last_;
 };
 
