@@ -22,50 +22,44 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <iostream>
 #include <vector>
 #include <string>
+#include <assert.h>
 #include "../basic/value.h"
 #include "../util/binary_buffer.h"
 #include "../util/text_buffer.h"
 #include "translated_position.h"
+#include "../util/interval.h"
 
 using std::vector;
 
 struct sequence
 {
-	static const char DELIMITER = '\x1f';
+	static constexpr Letter DELIMITER = DELIMITER_LETTER;
 	struct Reversed {};
 	struct Hardmasked {};
 	sequence():
 		len_ (0),
-		clipping_offset_ (0),
 		data_ (0)
 	{ }
-	sequence(const Letter *data, size_t len, int clipping_offset = 0):
+	sequence(const Letter *data, size_t len):
 		len_ (len),
-		clipping_offset_ (clipping_offset),
 		data_ (data)
 	{ }
+	sequence(const Letter *begin, const Letter *end) :
+		len_(end - begin),
+		data_(begin)
+	{}
 	sequence(const vector<Letter> &data):
 		len_(data.size()),
-		clipping_offset_(0),
 		data_(data.data())
 	{}
 	sequence(const sequence &seq, int from, int to):
 		len_(to-from+1),
-		clipping_offset_(0),
-		data_(&seq[from])
+		data_(seq.data() + from)
 	{}
 	size_t length() const
 	{
 		return len_;
 	}
-	size_t clipped_length() const
-	{
-		return len_ - clipping_offset_;
-	}
-	size_t aligned_clip(unsigned padding) const
-	{
-		return clipping_offset_ > (int)padding ? clipping_offset_ - padding : 0;
-	}
 	const Letter* data() const
 	{
 		return data_;
@@ -74,35 +68,41 @@ struct sequence
 	{
 		return data_ + len_;
 	}
-	const Letter* clipped_data() const
-	{
-		return data_ + clipping_offset_;
-	}
 	const Letter* aligned_data(unsigned padding) const
 	{
 		return data_ + padding;
 	}
-	const Letter& operator [](size_t i) const
+	Letter operator [](size_t i) const
 	{
+#ifdef SEQ_MASK
+		return data_[i] & LETTER_MASK;
+#else
 		return data_[i];
+#endif
 	}
 	bool empty() const
 	{ return len_ == 0; }
-	bool present() const {
-		return len_ > 0;
+	sequence operator+(int d) const {
+		return sequence(data_ + d, len_ - (size_t)d);
 	}
-	const char* c_str() const
-	{ return reinterpret_cast<const char*>(data_); }
 	size_t print(char *ptr, unsigned begin, unsigned len) const
 	{
 		for(unsigned i=begin;i<begin+len;++i)
+#ifdef SEQ_MASK
+			* (ptr++) = to_char(data_[i] & LETTER_MASK);
+#else
 			*(ptr++) = to_char(data_[i]);
+#endif
 		return len;
 	}
 	TextBuffer& print(TextBuffer &buf, size_t begin, size_t end, const Value_traits& value_traits) const
 	{
 		for (size_t i = begin; i < end; ++i)
+#ifdef SEQ_MASK
+			buf << value_traits.alphabet[(long)(data_[i] & LETTER_MASK)];
+#else
 			buf << value_traits.alphabet[(long)data_[i]];
+#endif
 		return buf;
 	}
 	std::ostream& print(std::ostream &os, const Value_traits &v) const
@@ -148,9 +148,6 @@ struct sequence
 	{
 		return sequence(*this, begin, end - 1);
 	}
-	std::string substr(int begin, int end) const {
-		return std::string(&data_[begin], &data_[end]);
-	}
 	friend TextBuffer& operator<<(TextBuffer &buf, const sequence &s)
 	{
 		return s.print(buf, value_traits);
@@ -174,6 +171,25 @@ struct sequence
 		}
 		return sequence(p + 1, s - p - 1);
 	}
+	std::vector<Letter> copy() const {
+		std::vector<Letter> v;
+		v.reserve(len_);
+		for (size_t i = 0; i < len_; ++i)
+			v.push_back(data_[i]);
+		return v;
+	}
+	void mask(const interval &i) {
+		for (int j = i.begin_; j < i.end_; ++j)
+			((Letter*)data_)[j] = value_traits.mask_char;
+	}
+	bool operator==(const sequence& s) const {
+		if (len_ != s.len_)
+			return false;
+		for (size_t i = 0; i < len_; ++i)
+			if (letter_mask(data_[i]) != letter_mask(s.data_[i]))
+				return false;
+		return true;
+	}
 	/*friend std::ostream& operator<<(std::ostream &os, const sequence &s)
 	{
 		std::cout << "co = " << s.clipping_offset_ << std::endl;
@@ -186,8 +202,7 @@ struct sequence
 	}*/
 	static vector<Letter> from_string(const char* str, const Value_traits &vt = value_traits);
 	size_t			len_;
-	int				clipping_offset_;
-	const Letter	*data_;
+	const Letter *data_;
 };
 
 struct TranslatedSequence
@@ -225,12 +240,12 @@ struct TranslatedSequence
 		return translated_[frame.index()];
 	}
 
-	const Letter& operator[](const TranslatedPosition &i) const
+	Letter operator[](const TranslatedPosition &i) const
 	{
 		return (*this)[i.frame][i];
 	}
 
-	const Letter& operator()(int in_strand, Strand strand) const
+	Letter operator()(int in_strand, Strand strand) const
 	{
 		assert(in_strand < (int)source_.length() - 2);
 		return translated_[in_strand % 3 + (strand == FORWARD ? 0 : 3)][in_strand / 3];
