@@ -1,6 +1,10 @@
 /****
 DIAMOND protein aligner
-Copyright (C) 2013-2017 Benjamin Buchfink <buchfink@gmail.com>
+Copyright (C) 2013-2020 Max Planck Society for the Advancement of Science e.V.
+                        Benjamin Buchfink
+                        Eberhard Karls Universitaet Tuebingen
+						
+Code developed by Benjamin Buchfink <benjamin.buchfink@tue.mpg.de>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -16,9 +20,7 @@ You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ****/
 
-#ifndef SHAPE_H_
-#define SHAPE_H_
-
+#pragma once
 #include <string.h>
 #include "const.h"
 #include "value.h"
@@ -29,62 +31,6 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "config.h"
 #include "translate.h"
 
-/*struct All_partitions {};
-struct Filter_partition {};
-
-template<typename _f>
-bool include_partition(unsigned p)
-{
-	return true;
-}
-
-template<>
-bool include_partition<Filter_partition>(unsigned p)
-{
-	return current_range.contains(p);
-}*/
-
-
-struct Letter_trail
-{
-	Letter_trail()
-	{
-		bucket[0] = 0;
-		for (int i = 1; i < 20; ++i)
-			bucket[i] = -1;
-	}
-	Letter_trail(const Reduction &reduction)
-	{
-		for (size_t i = 0; i < 20; ++i)
-			bucket[i] = reduction(i);
-	}
-	int operator()(char l) const
-	{
-		return bucket[(long)l];
-	}
-	int next() const
-	{
-		for (int i = 0; i < 20; ++i)
-			if (bucket[i] == -1)
-				return i;
-		return -1;
-	}
-	int buckets() const
-	{
-		int m = 0;
-		for (int i = 0; i < 20; ++i)
-			m = std::max(m, bucket[i]);
-		return m + 1;
-	}
-	double background_p() const;
-	double foreground_p(double id) const;
-	friend std::ostream& operator<<(std::ostream &s, const Letter_trail &t);
-	int bucket[20];
-};
-
-#define OPT_W 7
-typedef Letter_trail Trail[OPT_W];
-
 struct Shape
 {
 
@@ -104,7 +50,6 @@ struct Shape
 		id_ (id),
 		long_mask_(0)
 	{
-		assert(id < Const::max_shapes);
 		assert(strlen(code) <= 32);
 		memset(positions_, 0, sizeof(uint32_t)*Const::max_seed_weight);
 		const uint64_t b = Reduction::reduction.bit_size();
@@ -133,7 +78,10 @@ struct Shape
 #endif
 		for (unsigned i = 0; i < weight_; ++i) {
 			Letter l = seq[positions_[i]];
-			if (l == value_traits.mask_char || l == sequence::DELIMITER || l == Translator::STOP)
+#ifdef SEQ_MASK
+			l &= LETTER_MASK;
+#endif
+			if (!is_amino_acid(l))
 				return false;
 			unsigned r = Reduction::reduction(l);
 #ifdef FREQUENCY_MASKING
@@ -154,7 +102,10 @@ struct Shape
 		const uint64_t b = Reduction::reduction.bit_size();
 		for (unsigned i = 0; i < weight_; ++i) {
 			Letter l = seq[positions_[i]];
-			if (l == value_traits.mask_char || l == sequence::DELIMITER || l == Translator::STOP)
+#ifdef SEQ_MASK
+			l &= LETTER_MASK;
+#endif
+			if (l == value_traits.mask_char || l == sequence::DELIMITER || l == STOP_LETTER)
 				return false;
 			unsigned r = Reduction::reduction(l);
 			s <<= b;
@@ -168,6 +119,9 @@ struct Shape
 		s = 0;
 		for (unsigned i = 0; i < weight_; ++i) {
 			Letter l = seq[positions_[i]];
+#ifdef SEQ_MASK
+			l &= LETTER_MASK;
+#endif
 			if (l == value_traits.mask_char)
 				return false;
 			s *= Reduction::reduction.size();
@@ -180,6 +134,9 @@ struct Shape
 	{
 		for (unsigned i = 0; i < weight_; ++i) {
 			Letter l = seq[positions_[i]];
+#ifdef SEQ_MASK
+			l &= LETTER_MASK;
+#endif
 			if (l >= 20)
 				return false;
 			s[i] = l;
@@ -194,27 +151,6 @@ struct Shape
 		return s;
 	}
 
-	int score(const Letter *x, const Letter *y) const
-	{
-		int score = 0;
-		for (unsigned i = 0; i < weight_; ++i)
-			score += score_matrix(x[positions_[i]], y[positions_[i]]);
-		return score;
-	}
-
-	bool hit(const Letter *x, const Letter *y, const Trail &trail) const
-	{
-		for (unsigned i = 0; i < weight_; ++i)
-#if OPT_W==1
-			if (trail[0](x[positions_[i]]) != trail[0](y[positions_[i]]))
-				return false;
-#else
-			if (trail[i](x[positions_[i]]) != trail[i](y[positions_[i]]))
-				return false;
-#endif
-		return true;
-	}
-
 	bool contiguous() const
 	{
 		return length_ == weight_;
@@ -229,5 +165,3 @@ struct Shape
 	uint64_t long_mask_;
 
 };
-
-#endif /* SHAPE_H_ */
\ No newline at end of file
