@@ -18,6 +18,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 #include "seed_set.h"
 #include "../util/ptr_vector.h"
+#include "../util/math/integer.h"
 
 No_filter no_filter;
 
@@ -51,7 +52,7 @@ Seed_set::Seed_set(const Sequence_set &seqs, double max_coverage):
 		throw std::runtime_error("Contiguous seed required.");
 	PtrVector<Seed_set_callback> v;
 	v.push_back(new Seed_set_callback(data_, size_t(max_coverage*pow(Reduction::reduction.size(), shapes[0].length_))));
-	seqs.enum_seeds(v, seqs.partition(1), 0, 1, &no_filter);
+	seqs.enum_seeds(v, seqs.partition(1), 0, 1, &no_filter, true);
 	coverage_ = (double)v.back().coverage / pow(Reduction::reduction.size(), shapes[0].length_);
 }
 
