@@ -29,12 +29,13 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "../basic/shape_config.h"
 #include "../basic/seed_iterator.h"
 #include "../util/ptr_vector.h"
+#include "../basic/value.h"
 
 using std::cout;
 using std::endl;
 using std::pair;
 
-struct Sequence_set : public String_set<sequence::DELIMITER, 1>
+struct Sequence_set : public String_set<Letter, sequence::DELIMITER, 1>
 {
 
 	Sequence_set()
@@ -45,6 +46,11 @@ struct Sequence_set : public String_set<sequence::DELIMITER, 1>
 		verbose_stream << "Sequences = " << this->get_length() << ", letters = " << this->letters() << ", average length = " << this->avg_len() << endl;
 	}
 
+	sequence operator[](size_t i) const
+	{
+		return sequence(ptr(i), length(i));
+	}
+
 	pair<size_t, size_t> len_bounds(size_t min_len) const
 	{
 		const size_t l(this->get_length());
@@ -64,38 +70,6 @@ struct Sequence_set : public String_set<sequence::DELIMITER, 1>
 		return max;
 	}
 
-	sequence window_infix(size_t offset, unsigned &left) const
-	{
-		const Letter* begin(this->data(offset));
-		unsigned n(0);
-		while (*begin != sequence::DELIMITER && n <= config.window) {
-			--begin;
-			++n;
-		}
-		++begin;
-		left = config.window + 1 - n;
-		const Letter* end(this->data(offset));
-		n = 0;
-		while (*end != sequence::DELIMITER && n < config.window) {
-			++end;
-			++n;
-		}
-		return sequence(begin, end - begin);
-	}
-
-	sequence fixed_window_infix(size_t offset) const
-	{
-		const Letter* begin(this->data(offset));
-		unsigned n(0);
-		while (*begin != sequence::DELIMITER && n <= config.window) {
-			--begin;
-			++n;
-		}
-		++begin;
-		const Letter* s(this->data(offset - config.window));
-		return sequence(s, 2 * config.window, (int)(begin - s));
-	}
-
 	vector<size_t> partition(unsigned n_part) const
 	{
 		vector<size_t> v;
@@ -137,11 +111,11 @@ struct Sequence_set : public String_set<sequence::DELIMITER, 1>
 	}
 
 	template <typename _f, typename _filter>
-	void enum_seeds(PtrVector<_f> &f, const vector<size_t> &p, size_t shape_begin, size_t shape_end, const _filter *filter) const
+	void enum_seeds(PtrVector<_f> &f, const vector<size_t> &p, size_t shape_begin, size_t shape_end, const _filter *filter, bool contig = false) const
 	{
 		std::vector<std::thread> threads;
 		for (unsigned i = 0; i < f.size(); ++i)
-			threads.emplace_back(enum_seeds_worker<_f, _filter>, &f[i], this, (unsigned)p[i], (unsigned)p[i + 1], std::make_pair(shape_begin, shape_end), filter);
+			threads.emplace_back(enum_seeds_worker<_f, _filter>, &f[i], this, (unsigned)p[i], (unsigned)p[i + 1], std::make_pair(shape_begin, shape_end), filter, contig);
 		for (auto &t : threads)
 			t.join();
 	}
@@ -154,7 +128,7 @@ private:
 	template<typename _f, typename _filter>
 	void enum_seeds(_f *f, unsigned begin, unsigned end, pair<size_t, size_t> shape_range, const _filter *filter) const
 	{
-		vector<char> buf(max_len(begin, end));
+		vector<Letter> buf(max_len(begin, end));
 		uint64_t key;
 		for (unsigned i = begin; i < end; ++i) {
 			const sequence seq = (*this)[i];
@@ -219,10 +193,10 @@ private:
 	}
 
 	template<typename _f, typename _filter>
-	static void enum_seeds_worker(_f *f, const Sequence_set *seqs, unsigned begin, unsigned end, pair<size_t,size_t> shape_range, const _filter *filter)
+	static void enum_seeds_worker(_f *f, const Sequence_set *seqs, unsigned begin, unsigned end, pair<size_t,size_t> shape_range, const _filter *filter, bool contig)
 	{
 		static const char *errmsg = "Unsupported contiguous seed.";
-		if (shape_range.second - shape_range.first == 1 && shapes[shape_range.first].contiguous()) {
+		if (shape_range.second - shape_range.first == 1 && shapes[shape_range.first].contiguous() && shapes.count() == 1 && (config.algo == Config::query_indexed || contig)) {
 			const uint64_t b = Reduction::reduction.bit_size(), l = shapes[shape_range.first].length_;
 			switch (l) {
 			case 7:
