@@ -22,12 +22,14 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <vector>
 #include "../basic/sequence.h"
 #include "score_vector.h"
+#include "../basic/value.h"
+#include "comp_based_stats.h"
 
 using std::vector;
 
 #ifdef __SSE2__
 
-struct sequence_stream
+/*struct sequence_stream
 {
 	sequence_stream():
 		next (buffer_len),
@@ -85,7 +87,11 @@ struct sequence_stream
 			mask |= 1 << n;
 			return false;
 		}
-		*dest = *(src++) & 0x7f;
+#ifdef SEQ_MASK
+		*dest = *(src++) & LETTER_MASK;
+#else
+		*dest = *(src++);
+#endif
 		dest += 16/sizeof(_score);
 		return true;
 	}
@@ -95,27 +101,15 @@ struct sequence_stream
 	unsigned mask;
 };
 
+namespace DISPATCH_ARCH {
+
 template<typename _score>
 struct score_profile
 {
 
 	inline void set(const __m128i &seq)
 	{
-		assert(sizeof(data_)/sizeof(score_vector<_score>) >= value_traits.alphabet_size);
-		/*unsigned j = 0;
-		do {
-			data_[j] = score_vector<_score> (j, seq);
-			++j;
-			data_[j] = score_vector<_score> (j, seq);
-			++j;
-			data_[j] = score_vector<_score> (j, seq);
-			++j;
-			data_[j] = score_vector<_score> (j, seq);
-			++j;
-		} while(j<24);
-		data_[j] = score_vector<_score> (j, seq);
-		assert(j+1 == Value_traits<_val>::ALPHABET_SIZE);*/
-		for (unsigned j = 0; j < value_traits.alphabet_size; ++j)
+		for (unsigned j = 0; j < AMINO_ACID_COUNT; ++j)
 			data_[j] = score_vector<_score> (j, seq);
 	}
 
@@ -124,21 +118,38 @@ struct score_profile
 		return data_[(int)i];
 	}
 
-	score_vector<_score> data_[25];
+	score_vector<_score> data_[AMINO_ACID_COUNT];
 
 };
 
+}*/
+
 #endif
 
-struct Long_score_profile
+struct LongScoreProfile
 {
-	Long_score_profile()
+	LongScoreProfile()
 	{}
-	Long_score_profile(sequence seq)
+	LongScoreProfile(sequence seq, const Bias_correction &cbs)
 	{
-		for (unsigned l = 0; l < 25; ++l) {
-			const uint8_t *scores = &score_matrix.matrix8u()[l << 5];
-			data[l].reserve(seq.length() + 2*padding);
+		for (unsigned l = 0; l < AMINO_ACID_COUNT; ++l) {
+			const int8_t* scores = &score_matrix.matrix8()[l << 5];
+			data[l].reserve(seq.length() + 2 * padding);
+			data[l].insert(data[l].end(), padding, 0);
+			for (unsigned i = 0; i < seq.length(); ++i)
+				data[l].push_back(scores[(int)seq[i]] + cbs.int8[i]);
+			data[l].insert(data[l].end(), padding, 0);
+		}
+	}
+	LongScoreProfile(sequence seq)
+	{
+		set(seq);
+	}
+	void set(sequence seq) {
+		for (unsigned l = 0; l < AMINO_ACID_COUNT; ++l) {
+			const int8_t* scores = &score_matrix.matrix8()[l << 5];
+			data[l].clear();
+			data[l].reserve(seq.length() + 2 * padding);
 			data[l].insert(data[l].end(), padding, 0);
 			for (unsigned i = 0; i < seq.length(); ++i)
 				data[l].push_back(scores[(int)seq[i]]);
@@ -149,12 +160,12 @@ struct Long_score_profile
 	{
 		return data[0].size() - 2 * padding;
 	}
-	const uint8_t* get(Letter l, int i) const
+	const int8_t* get(Letter l, int i) const
 	{
 		return &data[(int)l][i + padding];
 	}
-	vector<uint8_t> data[25];
-	enum { padding = 32 };
+	vector<int8_t> data[AMINO_ACID_COUNT];
+	enum { padding = 128 };
 };
 
 #endif /* SCORE_PROFILE_H_ */
