@@ -22,7 +22,6 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <numeric>
 #include <atomic>
 #include "../dp.h"
-#include "swipe_matrix.h"
 #include "swipe.h"
 #include "target_iterator.h"
 #include "../../util/data_structures/mem_buffer.h"
@@ -360,7 +359,7 @@ Hsp traceback(sequence *query, Strand strand, int dna_len, const Banded3FrameSwi
 		}
 		else {
 			const pair<Edit_operation, int> g(it.walk_gap(d_begin, d1));
-			out.push_gap(g.first, g.second, &target.seq[it.j + g.second]);
+			out.push_gap(g.first, g.second, target.seq.data() + it.j + g.second);
 		}
 	}
 
@@ -413,7 +412,7 @@ list<Hsp> banded_3frame_swipe(
 		i0 = std::min(i0, i2 + 1 - band);
 	}
 
-	TargetIterator<ScoreTraits<_sv>::CHANNELS> targets(subject_begin, subject_end, i1, qlen, d_begin);
+	TargetIterator<Score> targets(subject_begin, subject_end, i1, qlen, d_begin);
 	Matrix dp(band * 3, targets.cols);
 
 	const _sv open_penalty(score_matrix.gap_open() + score_matrix.gap_extend()),
@@ -465,8 +464,8 @@ list<Hsp> banded_3frame_swipe(
 		}
 
 #ifdef DP_STAT
-		stat.net_cells += targets.live * (i1_ - i0_ + 1) * 3;
-		stat.gross_cells += ScoreTraits<_sv>::CHANNELS * (i1_ - i0_ + 1) * 3;
+		//stat.net_cells += targets.live * (i1_ - i0_ + 1) * 3;
+		//stat.gross_cells += ScoreTraits<_sv>::CHANNELS * (i1_ - i0_ + 1) * 3;
 #endif
 
 		Score col_best_[ScoreTraits<_sv>::CHANNELS];
@@ -509,7 +508,7 @@ list<Hsp> banded_3frame_swipe_targets(vector<DpTarget>::const_iterator begin,
 {
 	list<Hsp> out;
 	for (vector<DpTarget>::const_iterator i = begin; i < end; i += ScoreTraits<_sv>::CHANNELS) {
-		if (score_only || config.disable_traceback)
+		if (score_only || config.traceback_mode == TracebackMode::SCORE_ONLY)
 			out.splice(out.end(), banded_3frame_swipe<_sv, DP::ScoreOnly>(query, strand, i, i + std::min(vector<DpTarget>::const_iterator::difference_type(ScoreTraits<_sv>::CHANNELS), end - i), stat, parallel, overflow));
 		else
 			out.splice(out.end(), banded_3frame_swipe<_sv, DP::Traceback>(query, strand, i, i + std::min(vector<DpTarget>::const_iterator::difference_type(ScoreTraits<_sv>::CHANNELS), end - i), stat, parallel, overflow));
