@@ -19,7 +19,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "dp.h"
 #include "../basic/score_matrix.h"
 
-int xdrop_ungapped(const Letter *query, const Letter *subject, unsigned seed_len, unsigned &delta, unsigned &len)
+/*int xdrop_ungapped(const Letter *query, const Letter *subject, unsigned seed_len, unsigned &delta, unsigned &len)
 {
 	int score(0), st(0);
 	unsigned n(0);
@@ -32,7 +32,11 @@ int xdrop_ungapped(const Letter *query, const Letter *subject, unsigned seed_len
 		&& *q != sequence::DELIMITER
 		&& *s != sequence::DELIMITER)
 	{
+#ifdef SEQ_MASK
+		st += score_matrix(letter_mask(*q), letter_mask(*s));
+#else
 		st += score_matrix(*q, *s);
+#endif
 		score = std::max(score, st);
 		--q;
 		--s;
@@ -49,7 +53,11 @@ int xdrop_ungapped(const Letter *query, const Letter *subject, unsigned seed_len
 		&& *q != sequence::DELIMITER
 		&& *s != sequence::DELIMITER)
 	{
+#ifdef SEQ_MASK
+		st += score_matrix(letter_mask(*q), letter_mask(*s));
+#else
 		st += score_matrix(*q, *s);
+#endif
 		score = std::max(score, st);
 		++q;
 		++s;
@@ -57,11 +65,15 @@ int xdrop_ungapped(const Letter *query, const Letter *subject, unsigned seed_len
 	}
 
 	for (unsigned i = 0; i<seed_len; ++i)
+#ifdef SEQ_MASK
+		score += score_matrix(letter_mask(query[i]), letter_mask(subject[i]));
+#else
 		score += score_matrix(query[i], subject[i]);
+#endif
 
 	len = delta + n + seed_len;
 	return score;
-}
+}*/
 
 int xdrop_ungapped(const Letter *query, const Letter *subject, unsigned &delta, unsigned &len)
 {
@@ -73,7 +85,11 @@ int xdrop_ungapped(const Letter *query, const Letter *subject, unsigned &delta,
 		&& *q != sequence::DELIMITER
 		&& *s != sequence::DELIMITER)
 	{
+#ifdef SEQ_MASK
+		st += score_matrix(letter_mask(*q), letter_mask(*s));
+#else
 		st += score_matrix(*q, *s);
+#endif
 		if (st > score) {
 			score = st;
 			delta = n;
@@ -92,7 +108,11 @@ int xdrop_ungapped(const Letter *query, const Letter *subject, unsigned &delta,
 		&& *q != sequence::DELIMITER
 		&& *s != sequence::DELIMITER)
 	{
+#ifdef SEQ_MASK
+		st += score_matrix(letter_mask(*q), letter_mask(*s));
+#else
 		st += score_matrix(*q, *s);
+#endif
 		if (st > score) {
 			score = st;
 			len = n;
@@ -189,7 +209,6 @@ Diagonal_segment xdrop_ungapped(const sequence &query, const sequence &subject,
 	return Diagonal_segment(qa - delta, sa - delta, len + delta, score);
 }
 
-
 int xdrop_ungapped_right(const Letter *query, const Letter *subject, int &len)
 {
 	int score(0), st(0), n = 1;
@@ -202,7 +221,11 @@ int xdrop_ungapped_right(const Letter *query, const Letter *subject, int &len)
 		&& *q != sequence::DELIMITER
 		&& *s != sequence::DELIMITER)
 	{
+#ifdef SEQ_MASK
+		st += score_matrix(letter_mask(*q), letter_mask(*s));
+#else
 		st += score_matrix(*q, *s);
+#endif
 		if (st > score) {
 			score = st;
 			len = n;
@@ -212,4 +235,23 @@ int xdrop_ungapped_right(const Letter *query, const Letter *subject, int &len)
 		++n;
 	}
 	return score;
-}
\ No newline at end of file
+}
+
+int ungapped_window(const Letter* query, const Letter* subject, int window) {
+	int score = 0, st = 0, n = 0;
+	const Letter* q = query, * s = subject;
+	while (n < window)
+	{
+#ifdef SEQ_MASK
+		st += score_matrix(letter_mask(*q), *s);
+#else
+		st += score_matrix(*q, *s);
+#endif
+		st = std::max(st, 0);
+		score = std::max(score, st);
+		++q;
+		++s;
+		++n;
+	}
+	return score;
+}
