@@ -1,6 +1,10 @@
 /****
 DIAMOND protein aligner
-Copyright (C) 2013-2017 Benjamin Buchfink <buchfink@gmail.com>
+Copyright (C) 2013-2020 Max Planck Society for the Advancement of Science e.V.
+                        Benjamin Buchfink
+                        Eberhard Karls Universitaet Tuebingen
+						
+Code developed by Benjamin Buchfink <benjamin.buchfink@tue.mpg.de>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -37,9 +41,21 @@ struct JoinFetcher
 			files.push_back(new InputFile(**i));
 			query_ids.push_back(0);
 			files.back().read(&query_ids.back(), 1);
+
 		}
 		query_last = (unsigned)-1;
 	}
+
+	static void init(const vector<string> & tmp_file_names)
+	{
+		for (auto file_name : tmp_file_names) {
+			files.push_back(new InputFile(file_name, InputFile::NO_AUTODETECT));
+			query_ids.push_back(0);
+			files.back().read(&query_ids.back(), 1);
+		}
+		query_last = (unsigned)-1;
+	}
+
 	static void finish()
 	{
 		for (PtrVector<InputFile>::iterator i = files.begin(); i != files.end(); ++i)
@@ -47,13 +63,13 @@ struct JoinFetcher
 		files.clear();
 		query_ids.clear();
 	}
-	static unsigned next()
+	static uint32_t next()
 	{
 		return *std::min_element(query_ids.begin(), query_ids.end());
 	}
 	void fetch(unsigned b)
 	{
-		unsigned size;
+		uint32_t size;
 		files[b].read(&size, 1);
 		buf[b].clear();
 		buf[b].resize(size);
@@ -76,10 +92,10 @@ struct JoinFetcher
 		return next() != IntermediateRecord::FINISHED;
 	}
 	static PtrVector<InputFile> files;
-	static vector<unsigned> query_ids;
+	static vector<uint32_t> query_ids;
 	static unsigned query_last;
 	vector<BinaryBuffer> buf;
-	unsigned query_id, unaligned_from;
+	uint32_t query_id, unaligned_from;
 };
 
 PtrVector<InputFile> JoinFetcher::files;
@@ -145,12 +161,13 @@ struct BlockJoiner
 		}
 		std::make_heap(records.begin(), records.end());
 	}
-	bool get(vector<IntermediateRecord> &target_hsp)
+	bool get(vector<IntermediateRecord> &target_hsp, unsigned & block_idx)
 	{
 		if (records.empty())
 			return false;
 		const Join_record &first = records.front();
 		const unsigned block = first.block_;
+		block_idx = block;
 		const unsigned subject = first.info_.subject_id;
 		target_hsp.clear();
 		do {
@@ -171,16 +188,24 @@ struct BlockJoiner
 
 void join_query(vector<BinaryBuffer> &buf, TextBuffer &out, Statistics &statistics, unsigned query, const char *query_name, unsigned query_source_len, Output_format &f, const Metadata &metadata)
 {
-	const ReferenceDictionary& dict = ReferenceDictionary::get();
+	ReferenceDictionary& dict = ReferenceDictionary::get();
 	TranslatedSequence query_seq(get_translated_query(query));
 	BlockJoiner joiner(buf);
 	vector<IntermediateRecord> target_hsp;
 	unique_ptr<TargetCulling> culling(TargetCulling::get());
 
 	unsigned n_target_seq = 0;
-		
-	while (joiner.get(target_hsp)) {
-		const set<unsigned> rank_taxon_ids = config.taxon_k ? metadata.taxon_nodes->rank_taxid((*metadata.taxon_list)[dict.database_id(target_hsp.front().subject_id)], Rank::species) : set<unsigned>();
+	unsigned block_idx = 0;
+
+	while (joiner.get(target_hsp, block_idx)) {
+		ReferenceDictionary * dict_ptr;
+		if (config.multiprocessing) {
+			dict_ptr = & ReferenceDictionary::get(block_idx);
+		} else {
+			dict_ptr = & dict;
+		}
+
+		const set<unsigned> rank_taxon_ids = config.taxon_k ? metadata.taxon_nodes->rank_taxid((*metadata.taxon_list)[dict_ptr->database_id(target_hsp.front().subject_id)], Rank::species) : set<unsigned>();
 		const int c = culling->cull(target_hsp, rank_taxon_ids);
 		if (c == TargetCulling::FINISHED)
 			break;
@@ -197,13 +222,13 @@ void join_query(vector<BinaryBuffer> &buf, TextBuffer &out, Statistics &statisti
 					query,
 					query_seq,
 					query_name,
-					dict.check_id(i->subject_id),
-					dict.database_id(i->subject_id),
-					dict.name(i->subject_id),
-					dict.length(i->subject_id),
+					dict_ptr->check_id(i->subject_id),
+					dict_ptr->database_id(i->subject_id),
+					dict_ptr->name(i->subject_id),
+					dict_ptr->length(i->subject_id),
 					n_target_seq,
 					hsp_num,
-					config.use_lazy_dict ? dict.seq(i->subject_id) : sequence()
+					config.use_lazy_dict ? dict_ptr->seq(i->subject_id) : sequence()
 					).parse(), metadata, out);
 			}
 		}
@@ -221,19 +246,20 @@ void join_worker(Task_queue<TextBuffer, JoinWriter> *queue, const Parameters *pa
 	size_t n;
 	TextBuffer *out;
 	Statistics stat;
-	const String_set<0>& qids = query_ids::get();
+	const String_set<char, 0>& qids = query_ids::get();
 
 	while (queue->get(n, out, fetcher) && fetcher.query_id != IntermediateRecord::FINISHED) {
 		stat.inc(Statistics::ALIGNED);
 		size_t seek_pos;
 
-		const char * query_name = qids[qids.check_idx(fetcher.query_id)].c_str();
+		const char * query_name = qids[qids.check_idx(fetcher.query_id)];
+
 		const sequence query_seq = align_mode.query_translated ? query_source_seqs::get()[fetcher.query_id] : query_seqs::get()[fetcher.query_id];
 
 		if (*output_format != Output_format::daa && config.report_unaligned != 0) {
 			for (unsigned i = fetcher.unaligned_from; i < fetcher.query_id; ++i) {
-				output_format->print_query_intro(i, query_ids::get()[i].c_str(), get_source_query_len(i), *out, true);
-				output_format->print_query_epilog(*out, query_ids::get()[i].c_str(), true, *params);
+				output_format->print_query_intro(i, query_ids::get()[i], get_source_query_len(i), *out, true);
+				output_format->print_query_epilog(*out, query_ids::get()[i], true, *params);
 			}
 		}
 
@@ -250,21 +276,27 @@ void join_worker(Task_queue<TextBuffer, JoinWriter> *queue, const Parameters *pa
 			finish_daa_query_record(*out, seek_pos);
 		else
 			f->print_query_epilog(*out, query_name, false, *params);
-
 		queue->push(n);
 	}
 
 	statistics += stat;
 }
 
-void join_blocks(unsigned ref_blocks, Consumer &master_out, const PtrVector<TempFile> &tmp_file, const Parameters &params, const Metadata &metadata, DatabaseFile &db_file)
+void join_blocks(unsigned ref_blocks, Consumer &master_out, const PtrVector<TempFile> &tmp_file, const Parameters &params, const Metadata &metadata, DatabaseFile &db_file,
+	const vector<string> tmp_file_names)
 {
 	//ReferenceDictionary::get().init_rev_map();
 	task_timer timer("Building reference dictionary", 3);
 	if (config.use_lazy_dict)
 		ReferenceDictionary::get().build_lazy_dict(db_file);
 	timer.go("Joining output blocks");
-	JoinFetcher::init(tmp_file);
+
+	if (tmp_file_names.size() > 0) {
+		JoinFetcher::init(tmp_file_names);
+	} else {
+		JoinFetcher::init(tmp_file);
+	}
+
 	JoinWriter writer(master_out);
 	Task_queue<TextBuffer, JoinWriter> queue(3 * config.threads_, writer);
 	vector<thread> threads;
@@ -276,8 +308,8 @@ void join_blocks(unsigned ref_blocks, Consumer &master_out, const PtrVector<Temp
 	if (*output_format != Output_format::daa && config.report_unaligned != 0) {
 		TextBuffer out;
 		for (unsigned i = JoinFetcher::query_last + 1; i < query_ids::get().get_length(); ++i) {
-			output_format->print_query_intro(i, query_ids::get()[i].c_str(), get_source_query_len(i), out, true);
-			output_format->print_query_epilog(out, query_ids::get()[i].c_str(), true, params);
+			output_format->print_query_intro(i, query_ids::get()[i], get_source_query_len(i), out, true);
+			output_format->print_query_epilog(out, query_ids::get()[i], true, params);
 		}
 		writer(out);
 	}
