@@ -21,6 +21,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "tools.h"
 #include "../data/reference.h"
 #include "workflow.h"
+#include "../cluster/cluster_registry.h"
 #include "../util/simd.h"
 #ifdef EXTRA
 #include "../extra/compare.h"
@@ -45,19 +46,19 @@ void read_sim();
 void info();
 void seed_stat();
 void pairwise();
-void protein_snps();
 void translate();
 void filter_blasttab();
 void show_cbs();
 void reverse();
 void get_medoids_from_tree();
 void roc();
+void merge_tsv();
 
 void split();
 namespace Benchmark { DECL_DISPATCH(void, benchmark, ()) }
 namespace Util { namespace Algo { namespace UPGMA { void upgma(); } } }
 namespace Util { namespace Algo { namespace UPGMA_MC { void upgma(); } } }
-namespace Test { void run();
+namespace Test { int run();
 void simulate_seqs();
 void mutate();
 }
@@ -116,11 +117,8 @@ int main(int ac, const char* av[])
 		case Config::smith_waterman:
 			pairwise();
 			break;
-		case Config::protein_snps:
-			protein_snps();
-			break;
 		case Config::cluster:
-			Workflow::Cluster::run();
+			Workflow::Cluster::ClusterRegistry::get(config.cluster_algo)->run();
 			break;
 		case Config::translate:
 			translate();
@@ -147,7 +145,7 @@ int main(int ac, const char* av[])
 			Util::Algo::UPGMA_MC::upgma();
 			break;
 		case Config::regression_test:
-			Test::run();
+			return Test::run();
 			break;
 		case Config::reverse_seqs:
 			reverse();
@@ -161,6 +159,9 @@ int main(int ac, const char* av[])
 		case Config::roc:
 			roc();
 			break;
+		case Config::merge_tsv:
+			merge_tsv();
+			break;
 #ifdef EXTRA
 		case Config::compare:
 			compare();
