@@ -1,6 +1,9 @@
 /****
 DIAMOND protein aligner
-Copyright (C) 2013-2019 Benjamin Buchfink <buchfink@gmail.com>
+Copyright (C) 2016-2020 Max Planck Society for the Advancement of Science e.V.
+                        Benjamin Buchfink
+
+Code developed by Benjamin Buchfink <benjamin.buchfink@tue.mpg.de>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -16,15 +19,27 @@ You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ****/
 
-#ifndef SEARCH_H_
-#define SEARCH_H_
-
+#pragma once
 #include <stddef.h>
 #include "../util/simd.h"
-#include "../dp/dp.h"
+#include "../dp/ungapped.h"
 #include "../basic/shape_config.h"
 #include "../basic/statistics.h"
 #include "trace_pt_buffer.h"
+#include "../util/algo/pattern_matcher.h"
+#include "../util/data_structures/flat_array.h"
+#include "../util/scores/cutoff_table.h"
+#include "../basic/parameters.h"
+
+namespace Search {
+
+struct Context {
+	const PatternMatcher previous_matcher, current_matcher;
+	const Util::Scores::CutoffTable cutoff_table;
+	const int short_query_ungapped_cutoff;
+};
+
+}
 
 struct Stage1_hit
 {
@@ -46,30 +61,15 @@ struct Stage1_hit
 	unsigned q, s;
 };
 
-inline int stage2_ungapped(const Letter *query, const Letter *subject, unsigned sid, unsigned &delta, unsigned &len)
-{
-	return xdrop_ungapped(query, subject, shapes[sid].length_, delta, len);
-}
-
-void search_shape(unsigned sid, unsigned query_block, char *query_buffer, char *ref_buffer);
+void search_shape(unsigned sid, unsigned query_block, char *query_buffer, char *ref_buffer, const Parameters &params);
 bool use_single_indexed(double coverage, size_t query_letters, size_t ref_letters);
-void setup_search_params(pair<size_t, size_t> query_len_bounds, size_t chunk_db_letters);
 void setup_search();
 void setup_search_cont();
 
 namespace Search {
 
-DECL_DISPATCH(void, stage1, (const Packed_loc *q, size_t nq, const Packed_loc *s, size_t ns, Statistics &stats, Trace_pt_buffer::Iterator &out, const unsigned sid))
-DECL_DISPATCH(void, stage2, (\
-	const Packed_loc *q,\
-	const Packed_loc *s,\
-	const vector<Stage1_hit> &hits,\
-	Statistics &stats,\
-	Trace_pt_buffer::Iterator &out,\
-	const unsigned sid))
+DECL_DISPATCH(void, stage1, (const Packed_loc* q, size_t nq, const Packed_loc* s, size_t ns, Statistics& stats, Trace_pt_buffer::Iterator& out, const unsigned sid, const Context& context))
 
 }
 
-extern const double SINGLE_INDEXED_SEED_SPACE_MAX_COVERAGE;
-
-#endif
\ No newline at end of file
+extern const double SINGLE_INDEXED_SEED_SPACE_MAX_COVERAGE;
\ No newline at end of file
