@@ -1,6 +1,9 @@
 /****
 DIAMOND protein aligner
-Copyright (C) 2013-2019 Benjamin Buchfink <buchfink@gmail.com>
+Copyright (C) 2016-2020 Max Planck Society for the Advancement of Science e.V.
+                        Benjamin Buchfink
+						
+Code developed by Benjamin Buchfink <benjamin.buchfink@tue.mpg.de>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -30,7 +33,8 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "../util/data_structures/double_array.h"
 #include "../util/system/system.h"
 
-using namespace std;
+using std::vector;
+using std::atomic;
 
 Trace_pt_buffer* Trace_pt_buffer::instance;
 
@@ -54,26 +58,31 @@ void seed_join_worker(
 	}
 }
 
-void search_worker(atomic<unsigned> *seedp, const SeedPartitionRange *seedp_range, unsigned shape, size_t thread_id, DoubleArray<SeedArray::_pos> *query_seed_hits, DoubleArray<SeedArray::_pos> *ref_seed_hits)
+void search_worker(atomic<unsigned> *seedp, const SeedPartitionRange *seedp_range, unsigned shape, size_t thread_id, DoubleArray<SeedArray::_pos> *query_seed_hits, DoubleArray<SeedArray::_pos> *ref_seed_hits, const Search::Context *context)
 {
 	Trace_pt_buffer::Iterator* out = new Trace_pt_buffer::Iterator(*Trace_pt_buffer::instance, thread_id);
 	Statistics stats;
 	unsigned p;
 	while ((p = (*seedp)++) < seedp_range->end())
 		for (auto it = JoinIterator<SeedArray::_pos>(query_seed_hits[p].begin(), ref_seed_hits[p].begin()); it; ++it)
-			Search::stage1(it.r->begin(), it.r->size(), it.s->begin(), it.s->size(), stats, *out, shape);
+			Search::stage1(it.r->begin(), it.r->size(), it.s->begin(), it.s->size(), stats, *out, shape, *context);
 	delete out;
 	statistics += stats;
 }
 
-void search_shape(unsigned sid, unsigned query_block, char *query_buffer, char *ref_buffer)
+void search_shape(unsigned sid, unsigned query_block, char *query_buffer, char *ref_buffer, const Parameters &params)
 {
 	::partition<unsigned> p(Const::seedp, config.lowmem);
 	DoubleArray<SeedArray::_pos> query_seed_hits[Const::seedp], ref_seed_hits[Const::seedp];
 	log_rss();
 
 	for (unsigned chunk = 0; chunk < p.parts; ++chunk) {
-		message_stream << "Processing query block " << query_block << ", reference block " << current_ref_block << ", shape " << sid << ", index chunk " << chunk << '.' << endl;
+		message_stream << "Processing query block " << query_block + 1
+			<< ", reference block " << (current_ref_block + 1) << "/" << params.ref_blocks
+			<< ", shape " << (sid + 1) << "/" << shapes.count();
+		if (config.lowmem > 1)
+			message_stream << ", index chunk " << chunk + 1 << "/" << config.lowmem;
+		message_stream << '.' << endl;
 		const SeedPartitionRange range(p.getMin(chunk), p.getMax(chunk));
 		current_range = range;
 
@@ -91,7 +100,7 @@ void search_shape(unsigned sid, unsigned query_block, char *query_buffer, char *
 
 		timer.go("Computing hash join");
 		atomic<unsigned> seedp(range.begin());
-		vector<thread> threads;
+		vector<std::thread> threads;
 		for (size_t i = 0; i < config.threads_; ++i)
 			threads.emplace_back(seed_join_worker, query_idx, ref_idx, &seedp, &range, query_seed_hits, ref_seed_hits);
 		for (auto &t : threads)
@@ -100,15 +109,24 @@ void search_shape(unsigned sid, unsigned query_block, char *query_buffer, char *
 		timer.go("Building seed filter");
 		frequent_seeds.build(sid, range, query_seed_hits, ref_seed_hits);
 
+		Search::Context* context = nullptr;
+		const vector<uint32_t> patterns = shapes.patterns(0, sid + 1);
+		context = new Search::Context{ {patterns.data(), patterns.data() + patterns.size() - 1 },
+			{patterns.data(), patterns.data() + patterns.size() },
+			config.ungapped_evalue,
+			score_matrix.rawscore(config.short_query_ungapped_bitscore)
+		};
+
 		timer.go("Searching alignments");
 		seedp = range.begin();
 		threads.clear();
 		for (size_t i = 0; i < config.threads_; ++i)
-			threads.emplace_back(search_worker, &seedp, &range, sid, i, query_seed_hits, ref_seed_hits);
+			threads.emplace_back(search_worker, &seedp, &range, sid, i, query_seed_hits, ref_seed_hits, context);
 		for (auto &t : threads)
 			t.join();
 
 		delete ref_idx;
 		delete query_idx;
+		delete context;
 	}
 }
\ No newline at end of file
