@@ -36,17 +36,18 @@ void mcl() {
 
 void split() {
 	TextInputFile in(config.query_file);
-	vector<char> id, seq;
+	string id;
+	vector<Letter> seq;
 	size_t n = 0, f = 0, b = (size_t)(config.chunk_size * 1e9);
 	OutputFile *out = new OutputFile(std::to_string(f) + ".faa.gz", true);
-	while (FASTA_format().get_seq(id, seq, in)) {
+	while (FASTA_format().get_seq(id, seq, in, value_traits)) {
 		if (seq.size() + n > b) {
 			out->close();
 			delete out;
 			out = new OutputFile(std::to_string(++f) + ".faa.gz", true);
 			n = 0;
 		}
-		string blast_id = ::blast_id(string(id.data(), id.size()));
+		string blast_id = ::blast_id(id);
 		Util::Sequence::format(sequence(seq), blast_id.c_str(), nullptr, *out, "fasta", amino_acid_traits);
 		n += seq.size();
 	}
