@@ -1,6 +1,9 @@
 /****
 DIAMOND protein aligner
-Copyright (C) 2013-2018 Benjamin Buchfink <buchfink@gmail.com>
+Copyright (C) 2016-2020 Max Planck Society for the Advancement of Science e.V.
+                        Benjamin Buchfink
+						
+Code developed by Benjamin Buchfink <benjamin.buchfink@tue.mpg.de>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -16,13 +19,15 @@ You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ****/
 
-#ifndef DESERIALIZER_H_
-#define DESERIALIZER_H_
 
+#pragma once
 #include <vector>
 #include <utility>
+#include <iterator>
+#include <string.h>
 #include "stream_entity.h"
 #include "../algo/varint.h"
+#include "../system/endianness.h"
 
 struct DynamicRecordReader;
 
@@ -49,8 +54,10 @@ struct Deserializer
 	{
 		if (varint)
 			read_varint(*this, x);
-		else
+		else {
 			read(x);
+			x = big_endian_byteswap(x);
+		}
 		return *this;
 	}
 
@@ -63,12 +70,14 @@ struct Deserializer
 	Deserializer& operator>>(unsigned long &x)
 	{
 		read(x);
+		x = big_endian_byteswap(x);
 		return *this;
 	}
 
 	Deserializer& operator>>(unsigned long long &x)
 	{
 		read(x);
+		x = big_endian_byteswap(x);
 		return *this;
 	}
 
@@ -80,28 +89,30 @@ struct Deserializer
 
 	Deserializer& operator>>(std::string &s)
 	{
-		if (!read_until(s, '\0'))
+		s.clear();
+		if (!read_to(std::back_inserter(s), '\0'))
 			throw EndOfStream();
 		return *this;
 	}
 
 	Deserializer& operator>>(std::vector<std::string> &v)
 	{
-		int n;
+		uint32_t n;
+		varint = false;
 		*this >> n;
 		v.clear();
 		v.reserve(n);
 		std::string s;
-		for (int i = 0; i < n; ++i) {
+		for (uint32_t i = 0; i < n; ++i) {
 			*this >> s;
 			v.push_back(std::move(s));
 		}
 		return *this;
 	}
 
-	Deserializer& operator>>(std::vector<unsigned> &v)
+	Deserializer& operator>>(std::vector<uint32_t> &v)
 	{
-		unsigned n, x;
+		uint32_t n, x;
 		*this >> n;
 		v.clear();
 		v.reserve(n);
@@ -134,18 +145,33 @@ struct Deserializer
 		return begin_;
 	}
 
+	template<typename _it>
+	bool read_to(_it dst, char delimiter)
+	{
+		int d = delimiter;
+		do {
+			const char* p = (const char*)memchr((void*)begin_, d, avail());
+			if (p == 0) {
+				std::copy(begin_, end_, dst);
+			}
+			else {
+				const size_t n = p - begin_;
+				std::copy(begin_, begin_ + n, dst);
+				begin_ += n + 1;
+				return true;
+			}
+		} while (fetch());
+		return false;
+	}
+
 	size_t read_raw(char *ptr, size_t count);
-	bool read_until(std::string &dst, char delimiter);
-	bool read_until(std::vector<char> &dst, char delimiter);
 	DynamicRecordReader read_record();
 	~Deserializer();
 
 	bool varint;
 
 protected:
-
-	template<typename _t>
-	bool read_to(_t &container, char delimiter);
+	
 	void pop(char *dst, size_t n);
 	bool fetch();
 
@@ -157,5 +183,3 @@ protected:
 	StreamEntity *buffer_;
 	const char *begin_, *end_;
 };
-
-#endif
\ No newline at end of file
