@@ -1,6 +1,9 @@
 /****
 DIAMOND protein aligner
-Copyright (C) 2013-2018 Benjamin Buchfink <buchfink@gmail.com>
+Copyright (C) 2016-2020 Max Planck Society for the Advancement of Science e.V.
+                        Benjamin Buchfink
+						
+Code developed by Benjamin Buchfink <benjamin.buchfink@tue.mpg.de>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -16,23 +19,28 @@ You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ****/
 
-#ifndef INPUT_STREAM_BUFFER_H_
-#define INPUT_STREAM_BUFFER_H_
-
+#pragma once
 #include <utility>
+#include <memory>
+#include <thread>
 #include "stream_entity.h"
 
 struct InputStreamBuffer : public StreamEntity
 {
-	InputStreamBuffer(StreamEntity* prev);
-	virtual void rewind();
-	virtual void seek(size_t pos);
-	virtual void seek_forward(size_t n);
-	virtual pair<const char*, const char*> read();
+	enum { ASYNC = 4 };
+	InputStreamBuffer(StreamEntity* prev, int flags = 0);
+	virtual void rewind() override;
+	virtual void seek(size_t pos) override;
+	virtual void seek_forward(size_t n) override;
+	virtual pair<const char*, const char*> read() override;
+	virtual void putback(const char* p, size_t n) override;
+	virtual void close() override;
 private:
-	enum { BUF_SIZE = 4096 };
 
-	char buf_[BUF_SIZE];
-};
-
-#endif
\ No newline at end of file
+	static void load_worker(InputStreamBuffer *buf);
+	
+	std::unique_ptr<char[]> buf_, load_buf_;
+	size_t putback_count_, load_count_;
+	bool async_;
+	std::thread* load_worker_;
+};
\ No newline at end of file
