@@ -1,6 +1,10 @@
 /****
 DIAMOND protein aligner
-Copyright (C) 2013-2017 Benjamin Buchfink <buchfink@gmail.com>
+Copyright (C) 2013-2020 Max Planck Society for the Advancement of Science e.V.
+						Benjamin Buchfink
+						Eberhard Karls Universitaet Tuebingen
+
+Code developed by Benjamin Buchfink <benjamin.buchfink@tue.mpg.de>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -16,16 +20,13 @@ You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ****/
 
-#ifndef LOG_STREAM_H_
-#define LOG_STREAM_H_
-
+#pragma once
 #include <iostream>
 #include <fstream>
 #include <mutex>
 #include <limits.h>
 #include <chrono>
-
-using std::endl;
+#include <stdint.h>
 
 struct Message_stream
 {
@@ -86,7 +87,7 @@ struct task_timer
 	{
 		finish();
 	}
-	void go(const char *msg)
+	void go(const char* msg = nullptr)
 	{
 		finish();
 		start(msg);
@@ -96,17 +97,25 @@ struct task_timer
 	{
 		if (!msg_ || level_ == UINT_MAX)
 			return;
-		//if (print_ && !Cfg::debug_log)
-		get_stream() << " [" << get() << "s]" << endl;
-		/*else if (Cfg::debug_log) {
-			log_stream << '/' << msg_ << " [" << timer_.getElapsedTimeInSec() << "s]" << endl;
-		}*/
+		get_stream() << " [" << get() << "s]" << std::endl;
 		msg_ = 0;
 	}
 	double get()
 	{
 		return (double)std::chrono::duration_cast<std::chrono::milliseconds>(std::chrono::high_resolution_clock::now() - t).count() / 1000.0;
 	}
+	uint64_t seconds() const {
+		return (uint64_t)std::chrono::duration_cast<std::chrono::seconds>(std::chrono::high_resolution_clock::now() - t).count();
+	}
+	uint64_t milliseconds() const {
+		return (uint64_t)std::chrono::duration_cast<std::chrono::milliseconds>(std::chrono::high_resolution_clock::now() - t).count();
+	}
+	uint64_t microseconds() const {
+		return (uint64_t)std::chrono::duration_cast<std::chrono::microseconds>(std::chrono::high_resolution_clock::now() - t).count();
+	}
+	uint64_t nanoseconds() const {
+		return (uint64_t)std::chrono::duration_cast<std::chrono::nanoseconds>(std::chrono::high_resolution_clock::now() - t).count();
+	}
 	Message_stream& get_stream() const
 	{
 		switch (level_) {
@@ -123,21 +132,15 @@ struct task_timer
 private:
 	void start(const char *msg)
 	{
-		//if (print_ && !Cfg::debug_log) {
+		t = std::chrono::high_resolution_clock::now();
 		if (level_ == UINT_MAX)
 			return;
-		t = std::chrono::high_resolution_clock::now();
 		if (!msg)
 			return;
 		get_stream() << msg << "... " << std::flush;
-			//fflush(stdout);
-		/**}
-		else if (Cfg::debug_log)
-			log_stream << msg << "..." << endl;*/
+
 	}
 	unsigned level_;
 	const char *msg_;
 	std::chrono::high_resolution_clock::time_point t;
-};
-
-#endif
+};
\ No newline at end of file
