@@ -1,6 +1,25 @@
-#ifndef UTIL_STRING_TOKENIZER_H_
-#define UTIL_STRING_TOKENIZER_H_
+/****
+DIAMOND protein aligner
+Copyright (C) 2016-2020 Max Planck Society for the Advancement of Science e.V.
+						Benjamin Buchfink
 
+Code developed by Benjamin Buchfink <benjamin.buchfink@tue.mpg.de>
+
+This program is free software: you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation, either version 3 of the License, or
+(at your option) any later version.
+
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with this program.  If not, see <http://www.gnu.org/licenses/>.
+****/
+
+#pragma once
 #include <string>
 #include <string.h>
 #include <stdexcept>
@@ -56,7 +75,7 @@ struct Tokenizer {
 	}
 
 	Tokenizer& operator>>(long int &x) {
-		if (p == nullptr || *p == '\0')
+		if (!good())
 			throw TokenizerException();
 		char *end;
 		x = strtol(p, &end, 10);
@@ -79,8 +98,15 @@ struct Tokenizer {
 		return *this;
 	}
 
+	Tokenizer& operator>>(unsigned& x) {
+		long y;
+		*this >> y;
+		x = (unsigned)y;
+		return *this;
+	}
+
 	Tokenizer& operator>>(double &x) {
-		if (p == nullptr || *p == '\0')
+		if (!good())
 			throw TokenizerException("No token left");
 		char *end;
 		x = strtod(p, &end);
@@ -97,7 +123,7 @@ struct Tokenizer {
 	}
 
 	Tokenizer& operator>>(float &x) {
-		if (p == nullptr || *p == '\0')
+		if (!good())
 			throw TokenizerException("No token left");
 		char *end;
 		x = strtof(p, &end);
@@ -113,6 +139,15 @@ struct Tokenizer {
 		return *this;
 	}
 
+	bool good() const {
+		return p != nullptr && *p != '\0';
+	}
+
+	void skip_to(char c) {
+		const char* q = strchr(p, c);
+		p = q ? q + 1 : nullptr;
+	}
+
 private:
 	const char *p, *delimiter;
 	size_t len;
@@ -120,5 +155,3 @@ private:
 };
 
 }}
-
-#endif
