@@ -4,6 +4,10 @@
 #include <stdio.h>
 #include <string>
 
+enum class Color { RED, GREEN, YELLOW };
+
+void set_color(Color color, bool err = false);
+void reset_color(bool err = false);
 std::string executable_path();
 bool exists(const std::string &file_name);
 void auto_append_extension(std::string &str, const char *ext);
@@ -11,6 +15,8 @@ void auto_append_extension_if_exists(std::string &str, const char *ext);
 size_t getCurrentRSS();
 size_t getPeakRSS();
 void log_rss();
+size_t file_size(const char* name);
+double total_ram();
 
 #ifdef _MSC_VER
 #define POPEN _popen
