@@ -1,6 +1,10 @@
 /****
 DIAMOND protein aligner
-Copyright (C) 2013-2018 Benjamin Buchfink <buchfink@gmail.com>
+Copyright (C) 2013-2020 Max Planck Society for the Advancement of Science e.V.
+                        Benjamin Buchfink
+                        Eberhard Karls Universitaet Tuebingen
+						
+Code developed by Benjamin Buchfink <benjamin.buchfink@tue.mpg.de>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -16,9 +20,7 @@ You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ****/
 
-#ifndef UTIL_H_
-#define UTIL_H_
-
+#pragma once
 #include <vector>
 #include <algorithm>
 #include <iostream>
@@ -177,14 +179,6 @@ inline string* get_str(const char *s, const char *delimiters)
 	return new string (s, find_first_of(s, delimiters));
 }
 
-#ifdef __SSE2__
-inline __m128i _mm_set(char a)
-{
-	const int x = (int)a;
-	return _mm_set1_epi32(x << 24 | x << 16 | x << 8 | x);
-}
-#endif
-
 template<typename _t, unsigned d1, unsigned d2>
 struct Static_matrix
 {
@@ -314,14 +308,6 @@ private:
 	vector<_t> data_;
 };
 
-inline int get_idx(const char **a, size_t n, const char *s)
-{
-	for (size_t i = 0; i < n; ++i)
-		if (strcmp(a[i], s) == 0)
-			return (int)i;
-	return -1;
-}
-
 template<int n>
 inline int round_down(int x)
 {
@@ -471,14 +457,15 @@ inline double megabytes(size_t x)
 	return (double)x / (1 << 20);
 }
 
-inline int make_multiple(int x, int m)
+template<typename _t>
+inline _t make_multiple(_t x, _t m)
 {
-	return x % m == 0 ? x : x + m - x%m;
-}
-
-inline uint64_t next_power_of_2(double x)
-{
-	return 1llu << uint64_t(ceil(log(x) / log(2)));
+	if (x % m == 0)
+		return x;
+	_t d = m - x % m;
+	if (std::numeric_limits<_t>::max() - d < x)
+		return x;
+	return x + d;
 }
 
 inline string hex_print(const char *x, int len) {
@@ -518,6 +505,4 @@ std::vector<std::pair<_t1, _t2>> combine(const std::vector<_t1> &v1, const std::
 	for (size_t i = 0; i < v1.size(); ++i)
 		r.emplace_back(v1[i], v2[i]);
 	return r;
-}
-
-#endif /* UTIL_H_ */
+}
\ No newline at end of file
